@@ -12,24 +12,13 @@
 #
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
 import typing
 from typing import Any
 from typing import cast
-from typing import Optional
 
 import gc
 import logging
@@ -43,6 +32,7 @@ from gi.repository import Gdk
 from gi.repository import GLib
 from gi.repository import GObject
 from nbxmpp.idlequeue import IdleQueue
+from nbxmpp.protocol import JID
 
 import gajim
 from gajim.common import config as c_config
@@ -50,20 +40,21 @@ from gajim.common import configpaths
 from gajim.common import ged as ged_module
 from gajim.common import types
 from gajim.common.const import Display
-from gajim.common.i18n import LANG
+from gajim.common.i18n import get_default_lang
 
 if typing.TYPE_CHECKING:
     from gajim.common.call_manager import CallManager
     from gajim.common.cert_store import CertificateStore
     from gajim.common.commands import ChatCommands  # noqa: F401
     from gajim.common.preview import PreviewManager
-    from gajim.common.storage.archive import MessageArchiveStorage
+    from gajim.common.storage.archive.storage import MessageArchiveStorage
     from gajim.common.storage.cache import CacheStorage
     from gajim.common.storage.draft import DraftStorage
-    from gajim.common.storage.events import EventStorage
+    from gajim.common.storage.events.storage import EventStorage
     from gajim.common.task_manager import TaskManager
-    from gajim.gui.application import GajimApplication  # noqa: F401
-    from gajim.gui.main import MainWindow  # noqa: F401
+
+    from gajim.gtk.application import GajimApplication  # noqa: F401
+    from gajim.gtk.main import MainWindow  # noqa: F401
 
 
 interface = cast(types.InterfaceT, None)
@@ -76,6 +67,7 @@ bob_cache: dict[str, bytes] = {}
 app = None  # type: GajimApplication
 window = None  # type: MainWindow
 commands = None  # type: ChatCommands
+logging_records: list[str] = []
 
 ged = ged_module.GlobalEventsDispatcher()  # Global Events Dispatcher
 plugin_manager = cast(types.PluginManagerT, None)  # Plugins Manager
@@ -163,6 +155,10 @@ def is_portable() -> bool:
     return gajim.IS_PORTABLE
 
 
+def is_ms_store() -> bool:
+    return gajim.IS_MS_STORE
+
+
 def is_display(display: Display) -> bool:
     # XWayland reports as Display X11, so try with env var
     is_wayland = os.environ.get('XDG_SESSION_TYPE') == 'wayland'
@@ -204,7 +200,7 @@ def detect_dependencies() -> None:
         if _dependencies['GST'] and _dependencies['FARSTREAM']:
             conference = Gst.ElementFactory.make('fsrtpconference', None)
             conference.new_session(Farstream.MediaType.AUDIO)
-            from gajim.gui.gstreamer import create_gtk_widget
+            from gajim.gtk.gstreamer import create_gtk_widget
             gtk_widget = create_gtk_widget()
             if gtk_widget is not None:
                 _dependencies['AV'] = True
@@ -286,17 +282,20 @@ def detect_dependencies() -> None:
     try:
         import sentry_sdk  # noqa: F401
         _dependencies['SENTRY_SDK'] = True
-    except ImportError:
+    except Exception:
+        # Sentry has a lot of side effects on import
+        # make sure this optional dependency does not prevent
+        # Gajim from starting
         pass
 
     # Print results
     for dep, val in _dependencies.items():
         log('gajim').info('%-13s %s', dep, val)
 
-    log('gajim').info('Used language: %s', LANG)
+    log('gajim').info('Used language: %s', get_default_lang())
 
 
-def detect_desktop_env() -> Optional[str]:
+def detect_desktop_env() -> str | None:
     if sys.platform in ('win32', 'darwin'):
         return sys.platform
 
@@ -332,7 +331,7 @@ def get_jid_without_resource(jid: str) -> str:
 
 
 def get_number_of_connected_accounts(
-        accounts_list: Optional[list[str]] = None) -> int:
+        accounts_list: list[str] | None = None) -> int:
     '''
     Returns the number of connected accounts. You can optionally pass an
     accounts_list and if you do those will be checked, else all will be checked
@@ -379,8 +378,8 @@ def get_accounts_sorted() -> list[str]:
 
 
 def get_enabled_accounts_with_labels(
-        connected_only: bool = False,
-        private_storage_only: bool = False) -> list[list[str]]:
+    connected_only: bool = False
+) -> list[list[str]]:
     '''
     Returns a list with [account, account_label] entries.
     Order by account_label
@@ -389,8 +388,6 @@ def get_enabled_accounts_with_labels(
     for acc in connections:
         if connected_only and not account_is_connected(acc):
             continue
-        if private_storage_only and not account_supports_private_storage(acc):
-            continue
 
         accounts.append([acc, get_account_label(acc)])
 
@@ -402,12 +399,6 @@ def get_account_label(account: str) -> str:
     return settings.get_account_setting(account, 'account_label') or account
 
 
-def account_supports_private_storage(account: str) -> bool:
-    # If Delimiter module is not available we can assume
-    # Private Storage is not available
-    return connections[account].get_module('Delimiter').available
-
-
 def account_is_connected(account: str) -> bool:
     if account not in connections:
         return False
@@ -423,7 +414,7 @@ def account_is_available(account: str) -> bool:
 
 def get_transport_name_from_jid(
         jid: str,
-        use_config_setting: bool = True) -> Optional[str]:
+        use_config_setting: bool = True) -> str | None:
 
     '''
     Returns 'gg', 'irc' etc
@@ -470,20 +461,28 @@ def get_jid_from_account(account_name: str) -> str:
     '''
     Return the jid we use in the given account
     '''
-    name = settings.get_account_setting(account_name, 'name')
-    hostname = settings.get_account_setting(account_name, 'hostname')
-    return name + '@' + hostname
+    return settings.get_account_setting(account_name, 'address')
 
 
-def get_hostname_from_account(account_name: str, use_srv: bool = False) -> str:
-    '''
-    Returns hostname (if custom hostname is used, that is returned)
-    '''
-    if use_srv and connections[account_name].connected_hostname:
-        return connections[account_name].connected_hostname
-    if settings.get_account_setting(account_name, 'use_custom_host'):
-        return settings.get_account_setting(account_name, 'custom_host')
-    return settings.get_account_setting(account_name, 'hostname')
+def get_default_nick(account_name: str) -> str:
+    address = settings.get_account_setting(account_name, 'address')
+    jid = JID.from_string(address)
+    assert jid.localpart is not None
+    return jid.localpart
+
+
+def get_hostname_from_account(
+    account_name: str,
+    prefer_custom: bool = False
+) -> str:
+    if prefer_custom:
+        if settings.get_account_setting(account_name, 'use_custom_host'):
+            return settings.get_account_setting(account_name, 'custom_host')
+
+    address = settings.get_account_setting(account_name, 'address')
+    jid = JID.from_string(address)
+    assert jid.domain is not None
+    return jid.domain
 
 
 def get_priority(account: str, show: str) -> int:
@@ -517,7 +516,7 @@ def log(domain: str) -> logging.Logger:
 
 def load_css_config() -> None:
     global css_config   # pylint: disable=global-statement
-    from gajim.gui.css_config import CSSConfig
+    from gajim.gtk.css_config import CSSConfig
     css_config = CSSConfig()
 
 
@@ -537,7 +536,7 @@ def get_debug_mode() -> bool:
     return debug_enabled.exists()
 
 
-def get_stored_bob_data(algo_hash: str) -> Optional[bytes]:
+def get_stored_bob_data(algo_hash: str) -> bytes | None:
     try:
         return bob_cache[algo_hash]
     except KeyError:
