@@ -7,29 +7,17 @@
 #
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
 from typing import cast
-from typing import Generator
-from typing import Optional
-from typing import Union
 
 import importlib.resources
 import os
 import sys
 import tempfile
+from collections.abc import Generator
 from pathlib import Path
 
 from gi.repository import GLib
@@ -39,7 +27,7 @@ from gajim.common.const import PathLocation
 from gajim.common.const import PathType
 from gajim.common.i18n import _
 
-PathTupleT = tuple[Optional[PathLocation], Path, Optional[PathType]]
+PathTupleT = tuple[PathLocation | None, Path, PathType | None]
 
 
 def get(key: str) -> Path:
@@ -103,7 +91,7 @@ class ConfigPaths:
         self._paths: dict[str, PathTupleT] = {}
         self.profile = ''
         self.profile_separation = False
-        self.custom_config_root: Optional[Path] = None
+        self.custom_config_root: Path | None = None
 
         if os.name == 'nt':
             if gajim.IS_PORTABLE:
@@ -113,7 +101,7 @@ class ConfigPaths:
             else:
                 # Documents and Settings\[User Name]\Application Data\Gajim
                 self.config_root = self.cache_root = self.data_root = \
-                    Path(os.environ['appdata']) / 'Gajim'
+                    Path(os.environ['APPDATA']) / 'Gajim'
         else:
             self.config_root = Path(GLib.get_user_config_dir()) / 'gajim'
             self.cache_root = Path(GLib.get_user_cache_dir()) / 'gajim'
@@ -156,9 +144,9 @@ class ConfigPaths:
 
     def add(self,
             name: str,
-            path: Union[Path, str],
-            location: Optional[PathLocation] = None,
-            path_type: Optional[PathType] = None,
+            path: Path | str,
+            location: PathLocation | None = None,
+            path_type: PathType | None = None,
             unique: bool = False) -> None:
 
         path = Path(path)
@@ -219,6 +207,8 @@ class ConfigPaths:
             # Cache paths
             ('CACHE_DB', 'cache.db', PathLocation.CACHE, PathType.FILE),
             ('AVATAR', 'avatars', PathLocation.CACHE, PathType.FOLDER),
+            ('AVATAR_ICONS', 'avatar_icons',
+             PathLocation.CACHE, PathType.FOLDER),
             ('BOB', 'bob', PathLocation.CACHE, PathType.FOLDER),
 
             # Config paths
