@@ -13,45 +13,32 @@
 #
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
 from typing import Any
 from typing import TYPE_CHECKING
 
-import collections
-import copy
-import functools
 import hashlib
-import importlib.metadata
 import inspect
 import json
 import logging
 import os
-import platform
 import random
 import re
 import socket
 import string
 import sys
 import unicodedata
+import uuid
 import weakref
 import webbrowser
 from collections import defaultdict
 from collections.abc import Callable
 from datetime import datetime
 from datetime import timedelta
+from datetime import timezone
 from functools import wraps
 from pathlib import Path
 from string import Template
@@ -65,8 +52,6 @@ from cryptography.hazmat.backends import default_backend
 from gi.repository import GdkPixbuf
 from gi.repository import Gio
 from gi.repository import GLib
-from gi.repository import GObject
-from gi.repository import Soup
 from nbxmpp.const import Affiliation
 from nbxmpp.const import Chatstate
 from nbxmpp.const import ConnectionProtocol
@@ -79,8 +64,6 @@ from nbxmpp.protocol import Iq
 from nbxmpp.protocol import JID
 from nbxmpp.structs import CommonError
 from nbxmpp.structs import ProxyData
-from packaging.requirements import Requirement
-from packaging.version import Version as V
 from qrcode.image.pil import PilImage as QrcPilImage
 
 from gajim.common import app
@@ -562,21 +545,6 @@ def get_random_string(count: int = 16) -> str:
     return ''.join(random.choice(allowed) for char in range(count))
 
 
-@functools.lru_cache(maxsize=1)
-def get_os_info() -> str:
-    info = 'N/A'
-    if sys.platform in ('win32', 'darwin'):
-        info = f'{platform.system()} {platform.release()}'
-
-    elif sys.platform == 'linux':
-        try:
-            import distro
-            info = distro.name(pretty=True)
-        except ImportError:
-            info = platform.system()
-    return info
-
-
 def message_needs_highlight(text: str, nickname: str, own_jid: str) -> bool:
     '''
     Check whether 'text' contains 'nickname', 'own_jid', or any string of the
@@ -687,7 +655,7 @@ def get_subscription_request_msg(account: str | None = None) -> str:
     return Template(message).safe_substitute({'name': app.nicks[account]})
 
 
-def get_retraction_text(by: str | None,
+def get_moderation_text(by: str | JID | None,
                         reason: str | None) -> str:
 
     by_text = ''
@@ -717,6 +685,12 @@ def get_account_proxy(account: str, fallback=True) -> ProxyData | None:
 
 
 def get_proxy(proxy_name: str) -> ProxyData | None:
+    if proxy_name == 'no-proxy':
+        return ProxyData(type='direct',
+                         host='',
+                         username=None,
+                         password=None)
+
     try:
         settings = app.settings.get_proxy_settings(proxy_name)
     except ValueError:
@@ -752,12 +726,6 @@ def determine_proxy() -> ProxyData | None:
     return proxies[0] if proxies else None
 
 
-def version_condition(current_version: str, required_version: str) -> bool:
-    if V(current_version) < V(required_version):
-        return False
-    return True
-
-
 def call_counter(func):
     def helper(self, restart=False):
         if restart:
@@ -782,61 +750,6 @@ def load_json(path: Path,
     return json_dict.get(key, default)
 
 
-class AdditionalDataDict(collections.UserDict):
-    data: dict[str, Any]
-
-    @staticmethod
-    def _get_path_childs(full_path: str) -> list[str]:
-        if ':' in full_path:
-            return full_path.split(':')
-        return [full_path]
-
-    def set_value(self, full_path: str, key: str, value: str | None) -> None:
-        path_childs = self._get_path_childs(full_path)
-        _dict = self.data
-        for path in path_childs:
-            try:
-                _dict = _dict[path]
-            except KeyError:
-                _dict[path] = {}
-                _dict = _dict[path]
-        _dict[key] = value
-
-    def get_value(self,
-                  full_path: str,
-                  key: str,
-                  default: str | bool | None = None
-                  ) -> Any | None:
-
-        path_childs = self._get_path_childs(full_path)
-        _dict = self.data
-        for path in path_childs:
-            try:
-                _dict = _dict[path]
-            except KeyError:
-                return default
-        try:
-            return _dict[key]
-        except KeyError:
-            return default
-
-    def remove_value(self, full_path: str, key: str) -> None:
-        path_childs = self._get_path_childs(full_path)
-        _dict = self.data
-        for path in path_childs:
-            try:
-                _dict = _dict[path]
-            except KeyError:
-                return
-        try:
-            del _dict[key]
-        except KeyError:
-            return
-
-    def copy(self) -> AdditionalDataDict:
-        return copy.deepcopy(self)
-
-
 def delay_execution(milliseconds):
     # Delay the first call for `milliseconds`
     # ignore all other calls while the delay is active
@@ -1107,7 +1020,7 @@ def open_file_uri(uri: str) -> None:
         if sys.platform != 'win32':
             Gio.AppInfo.launch_default_for_uri(uri)
         else:
-            os.startfile(uri)  # noqa: S606
+            os.startfile(uri, 'open')  # noqa: S606
     except Exception as err:
         log.info("Couldn't open file URI %s: %s", uri, err)
 
@@ -1197,13 +1110,18 @@ def get_resource(account: str) -> str | None:
 def get_default_muc_config() -> dict[str, bool | str]:
     return {
         # XEP-0045 options
+        # https://xmpp.org/registrar/formtypes.html
+
         'muc#roomconfig_allowinvites': True,
-        'muc#roomconfig_publicroom': False,
+        'muc#roomconfig_allowpm': 'anyone',
+        'muc#roomconfig_changesubject': False,
+        'muc#roomconfig_enablelogging': False,
         'muc#roomconfig_membersonly': True,
-        'muc#roomconfig_persistentroom': True,
-        'muc#roomconfig_whois': 'anyone',
         'muc#roomconfig_moderatedroom': False,
-        'muc#roomconfig_enablelogging': True,
+        'muc#roomconfig_passwordprotectedroom': False,
+        'muc#roomconfig_persistentroom': True,
+        'muc#roomconfig_publicroom': False,
+        'muc#roomconfig_whois': 'moderators',
 
         # Ejabberd options
         'allow_voice_requests': False,
@@ -1211,7 +1129,7 @@ def get_default_muc_config() -> dict[str, bool | str]:
         'mam': True,
 
         # Prosody options
-        '{http://prosody.im/protocol/muc}roomconfig_allowmemberinvites': True,
+        '{http://prosody.im/protocol/muc}roomconfig_allowmemberinvites': False,
         'muc#roomconfig_enablearchiving': True,
     }
 
@@ -1289,7 +1207,7 @@ def is_role_change_allowed(self_contact: types.GroupchatParticipant,
     return self_contact.affiliation >= contact.affiliation
 
 
-def is_retraction_allowed(self_contact: types.GroupchatParticipant,
+def is_moderation_allowed(self_contact: types.GroupchatParticipant,
                           contact: types.GroupchatParticipant) -> bool:
 
     if self_contact.role < Role.MODERATOR:
@@ -1317,7 +1235,12 @@ class Observable:
 
             for handler in list(handlers):
                 func = handler()
-                if func is None or func.__self__ is obj:
+                # Don’t remove dead weakrefs from the handler list
+                # notify() will remove dead refs, and __disconnect()
+                # can be called from inside notify(), this can lead
+                # to race conditions where later notfiy tries to remove
+                # a dead ref which is not anymore in the list.
+                if func is not None and func.__self__ is obj:
                     handlers.remove(handler)
 
         if signals is None:
@@ -1355,6 +1278,10 @@ class Observable:
 
         weak_func = weakref.WeakMethod(func)
 
+        if weak_func in self._callbacks[signal_name]:
+            # Don’t register handler multiple times
+            return
+
         self._callbacks[signal_name].append(weak_func)
 
     def connect(self,
@@ -1457,7 +1384,7 @@ def get_custom_host(
     port = app.settings.get_account_setting(account, 'custom_port')
     type_ = app.settings.get_account_setting(account, 'custom_type')
 
-    if host.startswith('ws://') or host.startswith('wss://'):
+    if host.startswith(('ws://', 'wss://')):
         protocol = ConnectionProtocol.WEBSOCKET
     else:
         host = f'{host}:{port}'
@@ -1518,57 +1445,6 @@ def get_start_of_day(date_time: datetime) -> datetime:
                              microsecond=0)
 
 
-def get_os_name() -> str:
-    if sys.platform in ('win32', 'darwin'):
-        return platform.system()
-    if os.name == 'posix':
-        try:
-            import distro
-            return distro.name(pretty=True)
-        except ImportError:
-            return platform.system()
-    return ''
-
-
-def get_os_version() -> str:
-    if sys.platform in ('win32', 'darwin'):
-        return platform.version()
-    if os.name == 'posix':
-        try:
-            import distro
-            return distro.version(pretty=True)
-        except ImportError:
-            return platform.release()
-    return ''
-
-
-def get_gobject_version() -> str:
-    return '.'.join(map(str, GObject.pygobject_version))
-
-
-def get_glib_version() -> str:
-    return '.'.join(map(str, [GLib.MAJOR_VERSION,
-                              GLib.MINOR_VERSION,
-                              GLib.MICRO_VERSION]))
-
-
-def get_soup_version() -> str:
-    return '.'.join(map(str, [Soup.get_major_version(),
-                              Soup.get_minor_version(),
-                              Soup.get_micro_version()]))
-
-
-def package_version(requirement: str) -> bool:
-    req = Requirement(requirement)
-
-    try:
-        installed_version = importlib.metadata.version(req.name)
-    except importlib.metadata.PackageNotFoundError:
-        return False
-
-    return installed_version in req.specifier
-
-
 def make_path_from_jid(base_path: Path, jid: JID) -> Path:
     assert jid.domain is not None
     domain = jid.domain[:50]
@@ -1580,3 +1456,24 @@ def make_path_from_jid(base_path: Path, jid: JID) -> Path:
     if jid.resource is not None:
         return path / jid.resource[:30]
     return path
+
+
+def format_idle_time(idle_time: datetime) -> str:
+    now = datetime.now(timezone.utc)
+
+    now_date = now.date()
+    idle_date = idle_time.date()
+
+    if idle_date == now_date:
+        return idle_time.strftime(app.settings.get('time_format'))
+    if idle_date == now_date - timedelta(days=1):
+        return _('Yesterday (%s)') % idle_time.strftime(
+            app.settings.get('time_format'))
+    if idle_date >= now_date - timedelta(days=6):
+        return idle_time.strftime(f'%a {app.settings.get("time_format")}')
+
+    return idle_date.strftime(app.settings.get('date_format'))
+
+
+def get_uuid() -> str:
+    return str(uuid.uuid4())
