@@ -1,16 +1,6 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 # XEP-0363: HTTP File Upload
 
@@ -180,14 +170,9 @@ class HTTPUpload(BaseModule):
         if state.is_finished:
             uri = transfer.get_transformed_uri()
 
-            type_ = 'chat'
-            if transfer.is_groupchat:
-                type_ = 'groupchat'
-
             message = OutgoingMessage(account=transfer.account,
                                       contact=transfer.contact,
-                                      message=uri,
-                                      type_=type_,
+                                      text=uri,
                                       oob_url=uri)
 
             self._client.send_message(message)
@@ -317,6 +302,8 @@ class HTTPUpload(BaseModule):
             if request.get_error() == HTTPRequestError.CANCELLED:
                 self._log.info('Upload cancelled')
             else:
+                if not error:
+                    error = _('Upload could not be completed.')
                 transfer.set_error('http-response', error)
             return
 
@@ -343,7 +330,8 @@ class HTTPFileTransfer(FileTransfer):
     _errors = {
         'unsecure': _('The server returned an insecure transport (HTTP).'),
         'encryption-not-available': _('There is no encryption method available '
-                                      'for the chosen encryption.')
+                                      'for the chosen encryption.'),
+        'unknown': _('Unknown error.')
     }
 
     def __init__(self,
@@ -437,12 +425,14 @@ class HTTPFileTransfer(FileTransfer):
 
     def set_error(self, domain: str, text: str = '') -> None:
         if not text:
-            text = self._errors[domain]
+            text = self._errors.get(domain) or self._errors['unknown']
 
         super().set_error(domain, text)
+        self._cleanup()
 
     def set_finished(self) -> None:
         super().set_finished()
+        self._cleanup()
 
     def set_encrypted_data(self, data: bytes) -> None:
         self._temp_path.write_bytes(data)
@@ -455,3 +445,7 @@ class HTTPFileTransfer(FileTransfer):
         self.put_uri = result.put_uri
         self.get_uri = result.get_uri
         self._headers = result.headers
+
+    def _cleanup(self) -> None:
+        if self._temp_path.exists():
+            self._temp_path.unlink()
