@@ -1,17 +1,8 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
+from typing import Any
 from typing import Literal
 from typing import TypedDict
 
@@ -74,6 +65,7 @@ BoolSettings = Literal[
     'preview_verify_https',
     'print_status_in_chats',
     'remote_control',
+    'show_main_menu',
     'save_main_window_position',
     'send_on_ctrl_enter',
     'show_chatstate_in_banner',
@@ -83,6 +75,7 @@ BoolSettings = Literal[
     'show_notifications_away',
     'show_only_chat_and_online',
     'show_send_message_button',
+    'show_voice_message_button',
     'show_status_msgs_in_roster',
     'show_subject_on_join',
     'show_transports_group',
@@ -125,6 +118,10 @@ IntSettings = Literal[
     'preview_size',
 ]
 
+FloatSettings = Literal[
+    'app_font_size',
+]
+
 StringSettings = Literal[
     'action_on_close',
     'additional_uri_schemes',
@@ -159,10 +156,10 @@ StringSettings = Literal[
     'video_size',
 ]
 
-AllSettings = Literal[BoolSettings, IntSettings, StringSettings]
-AllSettingsT = str | int | bool | list[str]
+AllSettings = Literal[BoolSettings, IntSettings, FloatSettings, StringSettings]
+AllSettingsT = str | int | float | bool | list[str]
 
-APP_SETTINGS = {
+APP_SETTINGS: dict[str, str | int | float | bool | list[Any]] = {
     'additional_uri_schemes': '',
     'allow_open_file_uris': False,
     'always_english_wikipedia': False,
@@ -252,11 +249,13 @@ APP_SETTINGS = {
     'show_chatstate_in_banner': True,
     'show_help_start_chat': True,
     'show_in_taskbar': True,
+    'show_main_menu': True,
     'show_main_window_on_startup': 'always',
     'show_notifications': True,
     'show_notifications_away': False,
     'show_only_chat_and_online': False,
     'show_send_message_button': False,
+    'show_voice_message_button': True,
     'show_status_msgs_in_roster': True,
     'show_subject_on_join': True,
     'show_transports_group': True,
@@ -277,6 +276,7 @@ APP_SETTINGS = {
     'use_speller': True,
     'use_stun_server': False,
     'use_urgency_hint': True,
+    'app_font_size': 1.0,
     'video_framerate': '',
     'video_input_device': 'autovideosrc',
     'video_see_self': True,
@@ -321,6 +321,7 @@ BoolAccountSettings = Literal[
 StringAccountSettings = Literal[
     'account_color',
     'account_label',
+    'address',
     'attached_gpg_keys',
     'avatar_sha',
     'client_cert',
@@ -330,13 +331,11 @@ StringAccountSettings = Literal[
     'file_transfer_proxies',
     'filetransfer_preference',
     'gc_send_chatstate_default',
-    'hostname',
     'http_auth',
     'keyid',
     'keyname',
     'last_status',
     'last_status_msg',
-    'name',
     'password',
     'proxy',
     'resource',
@@ -415,6 +414,7 @@ ACCOUNT_SETTINGS = {
         'account_label': '',
         'active': False,
         'adjust_priority_with_status': False,
+        'address': '',
         'anonymous_auth': False,
         'answer_receipts': True,
         'attached_gpg_keys': '',
@@ -443,7 +443,6 @@ ACCOUNT_SETTINGS = {
         'gc_send_chatstate_default': 'composing_only',
         'gc_send_marker_private_default': True,
         'gc_send_marker_public_default': False,
-        'hostname': '',
         'http_auth': 'ask',
         'ignore_unknown_contacts': False,
         'is_zeroconf': False,
@@ -451,7 +450,6 @@ ACCOUNT_SETTINGS = {
         'keyname': '',
         'last_status': 'online',
         'last_status_msg': '',
-        'name': '',
         'password': '',
         'priority': 0,
         'proxy': '',
@@ -489,7 +487,7 @@ ACCOUNT_SETTINGS = {
     },
 
     'group_chat': {
-        'encryption': '',
+        'encryption': HAS_ACCOUNT_DEFAULT,
         'mute_until': '',
         'notify_on_all_messages': HAS_APP_DEFAULT,
         'print_join_left': HAS_APP_DEFAULT,
