@@ -1,34 +1,26 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
-
-import nbxmpp
-
-from gi.repository import Gdk
-from gi.repository import Gtk
-from gi.repository import GLib
-from gi.repository import GObject
-from gi.repository import Pango
+# SPDX-License-Identifier: GPL-3.0-only
 
 import cairo
+import nbxmpp
+from gi.repository import Gdk
+from gi.repository import Gtk
+from gi.repository import Pango
 
 from gajim.common import app
-from gajim.common.helpers import open_uri
-from gajim.common.i18n import _
+from gajim.common.const import ARTISTS
 from gajim.common.const import DEVS_CURRENT
 from gajim.common.const import DEVS_PAST
-from gajim.common.const import ARTISTS
 from gajim.common.const import THANKS
+from gajim.common.helpers import open_uri
+from gajim.common.i18n import _
+from gajim.common.util.app import get_extended_app_version
+from gajim.common.util.version import get_glib_version
+from gajim.common.util.version import get_gobject_version
+from gajim.common.util.version import get_soup_version
+
+from gajim.gtk.util import get_gtk_version
 
 
 class AboutDialog(Gtk.AboutDialog):
@@ -36,32 +28,25 @@ class AboutDialog(Gtk.AboutDialog):
         Gtk.AboutDialog.__init__(self)
         self.set_transient_for(app.window)
         self.set_name('Gajim')
-        self.set_version(app.version)
-        self.set_copyright('Copyright © 2003-2022 Gajim Team')
+        self.set_version(get_extended_app_version())
+        self.set_copyright('Copyright © 2003-2024 Gajim Team')
         self.set_license_type(Gtk.License.GPL_3_0_ONLY)
         self.set_website('https://gajim.org/')
 
-        gtk_ver = '%i.%i.%i' % (
-            Gtk.get_major_version(),
-            Gtk.get_minor_version(),
-            Gtk.get_micro_version())
-        gobject_ver = '.'.join(map(str, GObject.pygobject_version))
-        glib_ver = '.'.join(map(str, [GLib.MAJOR_VERSION,
-                                      GLib.MINOR_VERSION,
-                                      GLib.MICRO_VERSION]))
         cairo_ver = cairo.cairo_version_string()
         python_cairo_ver = cairo.version
 
         comments: list[str] = []
         comments.append(_('A fully-featured XMPP chat client'))
         comments.append('')
-        comments.append(_('GTK Version: %s') % gtk_ver)
-        comments.append(_('GLib Version: %s') % glib_ver)
+        comments.append(_('GTK Version: %s') % get_gtk_version())
+        comments.append(_('GLib Version: %s') % get_glib_version())
         comments.append(_('Pango Version: %s') % Pango.version_string())
-        comments.append(_('PyGObject Version: %s') % gobject_ver)
+        comments.append(_('PyGObject Version: %s') % get_gobject_version())
         comments.append(_('cairo Version: %s') % cairo_ver)
         comments.append(_('pycairo Version: %s') % python_cairo_ver)
         comments.append(_('python-nbxmpp Version: %s') % nbxmpp.__version__)
+        comments.append(_('libsoup Version: %s') % get_soup_version())
 
         self.set_comments('\n'.join(comments))
 
