@@ -1,40 +1,33 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
 from typing import cast
-from typing import Optional
+
+from functools import partial
 
 from gi.repository import Gio
 from gi.repository import GLib
 from gi.repository import GObject
 from gi.repository import Gtk
 from nbxmpp import JID
+from nbxmpp import Namespace
 
 from gajim.common import app
 from gajim.common import events
 from gajim.common import ged
 from gajim.common.ged import EventHelper
 from gajim.common.i18n import _
+from gajim.common.modules.contacts import GroupchatContact
 
-from . import structs
-from .chat_filter import ChatFilter
-from .chat_list import ChatList
-from .chat_list import ChatListRow
-from .dialogs import ConfirmationCheckDialog
-from .dialogs import DialogButton
+from gajim.gtk import structs
+from gajim.gtk.chat_filter import ChatFilter
+from gajim.gtk.chat_list import ChatList
+from gajim.gtk.chat_list_row import ChatListRow
+from gajim.gtk.dialogs import ConfirmationCheckDialog
+from gajim.gtk.dialogs import DialogButton
 
 
 class ChatListStack(Gtk.Stack, EventHelper):
@@ -51,7 +44,7 @@ class ChatListStack(Gtk.Stack, EventHelper):
                             ()),
         'chat-removed': (GObject.SignalFlags.RUN_LAST,
                          None,
-                         (str, object, str)),
+                         (str, object, str, bool)),
     }
 
     def __init__(self,
@@ -79,12 +72,11 @@ class ChatListStack(Gtk.Stack, EventHelper):
 
         self.register_events([
             ('message-received', ged.GUI2, self._on_event),
-            ('mam-message-received', ged.GUI2, self._on_event),
-            ('gc-message-received', ged.GUI2, self._on_event),
-            ('message-updated', ged.GUI2, self._on_event),
+            ('message-corrected', ged.GUI2, self._on_event),
             ('message-moderated', ged.GUI2, self._on_event),
             ('presence-received', ged.GUI2, self._on_event),
             ('message-sent', ged.GUI2, self._on_event),
+            ('message-deleted', ged.GUI2, self._on_event),
             ('file-request-received', ged.GUI2, self._on_event),
             ('jingle-request-received', ged.GUI2, self._on_event),
         ])
@@ -98,34 +90,35 @@ class ChatListStack(Gtk.Stack, EventHelper):
 
         for action in actions:
             action_name, variant, func = action
-            if variant is not None:
-                variant = GLib.VariantType.new(variant)
+            variant = GLib.VariantType.new(variant)
             act = Gio.SimpleAction.new(action_name, variant)
             act.connect('activate', func)
             app.window.add_action(act)
 
     def _on_visible_child_name(self, _stack: Gtk.Stack, _param: str) -> None:
-        if self._last_visible_child_name == self.get_visible_child_name():
+        new_visible_child_name = self.get_visible_child_name()
+        if self._last_visible_child_name == new_visible_child_name:
             return
 
         if self._last_visible_child_name != 'default':
             chat_list = cast(
-                ChatList,
+                ChatList | None,
                 self.get_child_by_name(self._last_visible_child_name))
-            chat_list.set_filter_text('')
-        last_child = self.get_visible_child_name() or 'default'
-        self._last_visible_child_name = last_child
+            if chat_list is not None:
+                chat_list.set_filter_text('')
+
+        self._last_visible_child_name = new_visible_child_name or 'default'
 
     def get_chatlist(self, workspace_id: str) -> ChatList:
         return self._chat_lists[workspace_id]
 
-    def get_selected_chat(self) -> Optional[ChatListRow]:
+    def get_selected_chat(self) -> ChatListRow | None:
         chat_list = self.get_current_chat_list()
         if chat_list is None:
             return None
         return chat_list.get_selected_chat()
 
-    def get_current_chat_list(self) -> Optional[ChatList]:
+    def get_current_chat_list(self) -> ChatList | None:
         workspace_id = self.get_visible_child_name()
         if workspace_id == 'empty' or workspace_id is None:
             return None
@@ -168,7 +161,7 @@ class ChatListStack(Gtk.Stack, EventHelper):
 
     def _on_row_selected(self,
                          _chat_list: ChatList,
-                         row: Optional[ChatListRow]
+                         row: ChatListRow | None
                          ) -> None:
         if row is None:
             self.emit('chat-unselected')
@@ -262,37 +255,72 @@ class ChatListStack(Gtk.Stack, EventHelper):
         chat_list = self._chat_lists[workspace_id]
         type_ = chat_list.get_chat_type(account, jid)
 
-        def _leave(not_ask_again: bool) -> None:
+        def _leave(not_ask_again: bool, unregister: bool = False) -> None:
             if not_ask_again:
                 app.settings.set('confirm_close_muc', False)
-            _remove()
+            _remove(unregister)
 
-        def _remove() -> None:
+        def _remove(unregister: bool = False) -> None:
             chat_list.remove_chat(account, jid, emit_unread=False)
             self.store_open_chats(workspace_id)
-            self.emit('chat-removed', account, jid, type_)
+            self.emit('chat-removed',
+                      account,
+                      jid,
+                      type_,
+                      unregister)
 
         if type_ != 'groupchat' or not app.settings.get('confirm_close_muc'):
             _remove()
             return
 
         client = app.get_client(account)
-        contact = client.get_module('Contacts').get_contact(jid, groupchat=True)
+        contact = client.get_module('Contacts').get_contact(jid)
+        assert isinstance(contact, GroupchatContact)
 
-        if contact.is_not_joined:
+        if contact.is_not_joined and client.state.is_available:
+            # For example a chat opened from the search bar
             _remove()
             return
 
+        muc_disco = contact.get_disco()
+        us = contact.get_self()
+        if muc_disco is None or us is None:
+            _remove()
+            return
+
+        buttons = [
+            DialogButton.make('Cancel'),
+            DialogButton.make(
+                'Accept', text=_('_Leave'), callback=_leave),
+        ]
+
+        affiliation = us.affiliation
+        text = _('By closing this chat, you will leave "%s".') % contact.name
+        if Namespace.REGISTER in muc_disco.features and not affiliation.is_none:
+            buttons.append(
+                DialogButton.make(
+                    'Delete',
+                    text=_('Leave _Permanently'),
+                    callback=partial(_leave, unregister=True),
+                )
+            )
+
+            text += '\n'
+            text += _('You may need an invite to join this group chat again, '
+                      'if you choose to leave it permanently.')
+            if affiliation.is_admin:
+                text += '\n'
+                text += _('Additionally, you will lose your administrator affiliation.')
+            elif affiliation.is_owner:
+                text += '\n'
+                text += _('Additionally, you will lose your owner affiliation.')
+
         ConfirmationCheckDialog(
             _('Leave Group Chat'),
             _('Are you sure you want to leave this group chat?'),
-            _('If you close this chat, you will leave '
-              '"%s".') % contact.name,
+            text,
             _('_Do not ask me again'),
-            [DialogButton.make('Cancel'),
-             DialogButton.make('Accept',
-                               text=_('_Leave'),
-                               callback=_leave)],
+            buttons,
             transient_for=app.window).show()
 
     def remove_chats_for_account(self, account: str) -> None:
@@ -300,14 +328,14 @@ class ChatListStack(Gtk.Stack, EventHelper):
             chat_list.remove_chats_for_account(account)
             self.store_open_chats(workspace_id)
 
-    def find_chat(self, account: str, jid: JID) -> Optional[ChatList]:
+    def find_chat(self, account: str, jid: JID) -> ChatList | None:
         for chat_list in self._chat_lists.values():
             if chat_list.contains_chat(account, jid):
                 return chat_list
         return None
 
     def contains_chat(self, account: str, jid: JID,
-                      workspace_id: Optional[str] = None) -> bool:
+                      workspace_id: str | None = None) -> bool:
         if workspace_id is None:
             return any(chat_list.contains_chat(account, jid) for
                        chat_list in self._chat_lists.values())
@@ -325,7 +353,7 @@ class ChatListStack(Gtk.Stack, EventHelper):
                               account: str,
                               jid: JID,
                               include_silent: bool = False
-                              ) -> Optional[int]:
+                              ) -> int | None:
         for chat_list in self._chat_lists.values():
             count = chat_list.get_chat_unread_count(
                 account, jid, include_silent)
