@@ -1,16 +1,6 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 # Constants for the gtk module
 
@@ -147,6 +137,7 @@ WINDOW_MODULES = {
     'AddContact': 'gajim.gtk.add_contact',
     'AdHocCommands': 'gajim.gtk.adhoc',
     'AdvancedConfig': 'gajim.gtk.advanced_config',
+    'DBMigration': 'gajim.gtk.db_migration',
     'BlockingList': 'gajim.gtk.blocking',
     'Bookmarks': 'gajim.gtk.bookmarks',
     'CertificateDialog': 'gajim.gtk.certificate_dialog',
@@ -155,7 +146,7 @@ WINDOW_MODULES = {
     'CreateGroupchatWindow': 'gajim.gtk.groupchat_creation',
     'Features': 'gajim.gtk.features',
     'GroupchatDetails': 'gajim.gtk.groupchat_details',
-    'GroupChatInvitation': 'gajim.gtk.groupchat_invitation',
+    'GroupChatInvitationDialog': 'gajim.gtk.groupchat_invitation',
     'GroupchatJoin': 'gajim.gtk.groupchat_join',
     'HistoryExport': 'gajim.gtk.history_export',
     'HistorySyncAssistant': 'gajim.gtk.history_sync',
@@ -177,7 +168,7 @@ WINDOW_MODULES = {
     'SynchronizeAccounts': 'gajim.gtk.synchronize_accounts',
     'Themes': 'gajim.gtk.themes',
     'WorkspaceDialog': 'gajim.gtk.workspace_dialog',
-    'XMLConsoleWindow': 'gajim.gtk.xml_console',
+    'DebugConsoleWindow': 'gajim.gtk.debug_console',
 }
 
 
@@ -218,11 +209,15 @@ MAIN_WIN_ACTIONS = [
     ('input-clear', None, True),
     ('show-emoji-chooser', None, True),
     ('activate-message-selection', 'u', True),
-    ('delete-message-locally', 'u', True),
+    ('delete-message-locally', 'a{sv}', True),
     ('correct-message', None, False),
     ('copy-message', 's', True),
-    ('retract-message', 'a{sv}', False),
+    ('moderate-message', 'a{sv}', False),
     ('quote', 's', False),
+    ('quote-next', None, True),
+    ('quote-prev', None, True),
+    ('reply', 'u', False),
+    ('jump-to-message', 'au', True),
     ('mention', 's', False),
     ('send-file-httpupload', 'as', False),
     ('send-file-jingle', 'as', False),
@@ -270,7 +265,11 @@ MAIN_WIN_ACTIONS = [
     ('switch-workspace-7', None, True),
     ('switch-workspace-8', None, True),
     ('switch-workspace-9', None, True),
+    ('increase-app-font-size', None, True),
+    ('decrease-app-font-size', None, True),
+    ('reset-app-font-size', None, True),
     ('toggle-chat-list', None, True),
+    ('toggle-menu-bar', None, True),
     ('add-workspace', 's', True),
     ('edit-workspace', 's', True),
     ('remove-workspace', 's', True),
@@ -293,7 +292,7 @@ ACCOUNT_ACTIONS = [
     ('contact-info', 's'),
     ('block-contact', 's'),
     ('remove-contact', 's'),
-    ('execute-command', 's'),
+    ('execute-command', 'as'),
     ('modify-gateway', 's'),
     ('archive', 's'),
     ('blocking', 's'),
