@@ -1,33 +1,22 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
 from typing import Any
 from typing import cast
-from typing import Optional
-from typing import Union
 
+import datetime as dt
 import logging
 import time
+from collections.abc import Sequence
 
 from gi.repository import Gio
 from gi.repository import GLib
 from gi.repository import Gtk
 from nbxmpp import JID
 from nbxmpp.const import StatusCode
-from nbxmpp.modules.security_labels import Displaymarking
 from nbxmpp.structs import MucSubject
 
 from gajim.common import app
@@ -35,26 +24,26 @@ from gajim.common import events
 from gajim.common import ged
 from gajim.common import helpers
 from gajim.common import types
-from gajim.common.const import KindConstant
+from gajim.common.const import XmppUriQuery
 from gajim.common.ged import EventHelper
-from gajim.common.helpers import AdditionalDataDict
-from gajim.common.helpers import get_retraction_text
-from gajim.common.helpers import to_user_string
+from gajim.common.helpers import get_moderation_text
 from gajim.common.i18n import _
 from gajim.common.modules.contacts import BareContact
 from gajim.common.modules.contacts import GroupchatContact
 from gajim.common.modules.contacts import GroupchatParticipant
 from gajim.common.modules.httpupload import HTTPFileTransfer
-from gajim.common.storage.archive import ConversationRow
+from gajim.common.storage.archive.const import ChatDirection
+from gajim.common.storage.archive.models import Message
 
 from gajim.gtk.builder import get_builder
 from gajim.gtk.conversation.jump_to_end_button import JumpToEndButton
 from gajim.gtk.conversation.message_selection import MessageSelection
+from gajim.gtk.conversation.rows.widgets import MessageRowActions
 from gajim.gtk.conversation.view import ConversationView
 from gajim.gtk.groupchat_roster import GroupchatRoster
 from gajim.gtk.groupchat_state import GroupchatState
 
-HistoryRowT = Union[events.ApplicationEvent, ConversationRow]
+HistoryRowT = events.ApplicationEvent | Message
 
 REQUEST_LINES_COUNT = 20
 
@@ -71,7 +60,10 @@ class ChatControl(EventHelper):
 
         self._ui = get_builder('chat_control.ui')
 
-        self._scrolled_view = ConversationView()
+        self._message_row_actions = MessageRowActions()
+        self._ui.conv_view_overlay.add_overlay(self._message_row_actions)
+
+        self._scrolled_view = ConversationView(self._message_row_actions)
         self._scrolled_view.connect('autoscroll-changed',
                                     self._on_autoscroll_changed)
         self._scrolled_view.connect('request-history', self._request_history)
@@ -90,14 +82,15 @@ class ChatControl(EventHelper):
         self._ui.conv_view_overlay.add_overlay(self._jump_to_end_button)
 
         self._roster = GroupchatRoster()
-
-        self._ui.conv_view_box.add(self._roster)
+        self._ui.conv_view_paned.pack2(self._roster, False, False)
 
         # Used with encryption plugins
         self.sendmessage = False
 
         app.window.get_action('activate-message-selection').connect(
             'activate', self._on_activate_message_selection)
+        app.window.get_action('jump-to-message').connect(
+            'activate', self._on_jump_to_message)
 
         self.widget = cast(Gtk.Box, self._ui.get_object('control_box'))
         self.widget.show_all()
@@ -124,6 +117,18 @@ class ChatControl(EventHelper):
         assert self._client is not None
         return self._client
 
+    @property
+    def is_chat(self) -> bool:
+        return isinstance(self.contact, BareContact)
+
+    @property
+    def is_privatechat(self) -> bool:
+        return isinstance(self.contact, GroupchatParticipant)
+
+    @property
+    def is_groupchat(self) -> bool:
+        return isinstance(self.contact, GroupchatContact)
+
     def is_loaded(self, account: str, jid: JID) -> bool:
         if self._contact is None:
             return False
@@ -135,6 +140,23 @@ class ChatControl(EventHelper):
     def get_group_chat_roster(self) -> GroupchatRoster:
         return self._roster
 
+    def get_conversation_view(self) -> ConversationView:
+        return self._scrolled_view
+
+    def add_command_output(self, text: str, is_error: bool) -> None:
+        self._scrolled_view.add_command_output(text, is_error)
+
+    def add_info_message(self,
+                         text: str,
+                         timestamp: dt.datetime | None = None
+                         ) -> None:
+
+        self._scrolled_view.add_info_message(text, timestamp)
+
+    def drag_data_file_transfer(self, selection: Gtk.SelectionData) -> None:
+        app.window.activate_action('send-file',
+                                   GLib.Variant('as', selection.get_uris()))
+
     def clear(self) -> None:
         log.info('Clear')
 
@@ -148,9 +170,51 @@ class ChatControl(EventHelper):
         self._roster.clear()
         self.unregister_events()
 
-    def switch_contact(self, contact: Union[BareContact,
-                                            GroupchatContact,
-                                            GroupchatParticipant]) -> None:
+    def remove_message(self, pk: int) -> None:
+        self._scrolled_view.remove_message(pk)
+
+    def reset_view(self) -> None:
+        self._scrolled_view.reset()
+
+    def get_autoscroll(self) -> bool:
+        return self._scrolled_view.get_autoscroll()
+
+    def scroll_to_message(
+        self,
+        pk: int,
+        timestamp: dt.datetime
+    ) -> None:
+        row = self._scrolled_view.get_row_by_pk(pk)
+        if row is None:
+            m = app.storage.archive.get_message_with_pk(pk)
+            if m is None:
+                log.info('Message with pk %s was not found in DB', pk)
+                return
+
+            # Clear view and reload conversation around timestamp
+            self._scrolled_view.reset()
+            self._scrolled_view.block_signals(True)
+            messages: list[Message] = []
+            messages.append(m)
+            messages.extend(app.storage.archive.get_conversation_before_after(
+                self.contact.account, self.contact.jid, True, timestamp, 50))
+            messages.extend(app.storage.archive.get_conversation_before_after(
+                self.contact.account, self.contact.jid, False, timestamp, 50))
+            self._add_messages(messages)
+            self._scrolled_view.set_history_complete(False, False)
+
+        GLib.idle_add(self._scrolled_view.block_signals, False)
+        GLib.idle_add(
+            self._scrolled_view.scroll_to_message_and_highlight,
+            pk)
+
+    def mark_as_read(self) -> None:
+        self._jump_to_end_button.reset_unread_count()
+
+    def switch_contact(
+        self,
+        contact: BareContact | GroupchatContact | GroupchatParticipant
+    ) -> None:
 
         log.info('Switch to %s (%s)', contact.jid, contact.account)
         if self._contact is not None:
@@ -161,7 +225,9 @@ class ChatControl(EventHelper):
         self._client = app.get_client(contact.account)
 
         self._jump_to_end_button.switch_contact(contact)
+        self._message_row_actions.switch_contact(contact)
         self._scrolled_view.switch_contact(contact)
+        self._request_history(None, True)
         self._groupchat_state.switch_contact(contact)
         self._roster.switch_contact(contact)
 
@@ -190,6 +256,7 @@ class ChatControl(EventHelper):
                 'room-config-changed': self._on_room_config_changed,
                 'room-presence-error': self._on_room_presence_error,
                 'room-subject': self._on_room_subject,
+                'room-affiliation-changed': self._on_room_affiliation_changed,
             })
 
         self._client.get_module('Chatstate').set_active(contact)
@@ -198,15 +265,14 @@ class ChatControl(EventHelper):
             contact)
         if transfers is not None:
             for transfer in transfers:
-                self.add_file_transfer(transfer)
+                self._add_file_transfer(transfer)
 
         if isinstance(contact, GroupchatContact):
             if (not app.settings.get('show_subject_on_join') or
                     contact.is_joining):
                 return
 
-            muc_data = self._client.get_module('MUC').get_muc_data(
-                str(contact.jid))
+            muc_data = self._client.get_module('MUC').get_muc_data(contact.jid)
             if muc_data is not None and muc_data.subject is not None:
                 self._scrolled_view.add_muc_subject(
                     muc_data.subject, muc_data.last_subject_timestamp)
@@ -218,13 +284,14 @@ class ChatControl(EventHelper):
         self.register_events([
             ('presence-received', ged.GUI2, self._on_presence_received),
             ('message-sent', ged.GUI2, self._on_message_sent),
+            ('message-deleted', ged.GUI2, self._on_message_deleted),
+            ('message-acknowledged', ged.GUI2, self._on_message_acknowledged),
             ('message-received', ged.GUI2, self._on_message_received),
-            ('mam-message-received', ged.GUI2, self._on_mam_message_received),
-            ('gc-message-received', ged.GUI2, self._on_gc_message_received),
-            ('message-updated', ged.GUI2, self._on_message_updated),
+            ('message-corrected', ged.GUI2, self._on_message_corrected),
             ('message-moderated', ged.GUI2, self._on_message_moderated),
             ('receipt-received', ged.GUI2, self._on_receipt_received),
             ('displayed-received', ged.GUI2, self._on_displayed_received),
+            ('reaction-updated', ged.GUI2, self._on_reaction_updated),
             ('message-error', ged.GUI2, self._on_message_error),
             ('call-stopped', ged.GUI2, self._on_call_stopped),
             ('jingle-request-received',
@@ -233,6 +300,7 @@ class ChatControl(EventHelper):
             ('file-request-sent', ged.GUI2, self._on_file_request_event),
             ('http-upload-started', ged.GUI2, self._on_http_upload_started),
             ('http-upload-error', ged.GUI2, self._on_http_upload_error),
+            ('encryption-check', ged.GUI2, self._on_encryption_info),
         ])
 
     def _is_event_processable(self, event: Any) -> bool:
@@ -254,8 +322,9 @@ class ChatControl(EventHelper):
             return
 
         contact = self.client.get_module('Contacts').get_contact(event.fjid)
-        if isinstance(contact, BareContact):
+        if isinstance(contact, BareContact | GroupchatContact):
             return
+
         self._scrolled_view.add_user_status(self.contact.name,
                                             contact.show.value,
                                             contact.status)
@@ -264,137 +333,50 @@ class ChatControl(EventHelper):
         if not self._is_event_processable(event):
             return
 
-        if not event.message:
+        self._add_message(event.message)
+
+    def _on_message_deleted(self, event: events.MessageDeleted) -> None:
+        if not self._is_event_processable(event):
             return
 
-        if self.contact.is_groupchat:
+        self.remove_message(event.pk)
+
+    def _on_message_acknowledged(
+        self,
+        event: events.MessageAcknowledged
+    ) -> None:
+        if not self._is_event_processable(event):
             return
 
-        message_id = event.message_id
-
-        if event.label:
-            displaymarking = event.label.displaymarking
-        else:
-            displaymarking = None
-
-        if event.correct_id:
-            self._scrolled_view.correct_message(
-                event.correct_id, event.message, self.get_our_nick())
-            return
-
-        self.add_message(event.message,
-                         'outgoing',
-                         tim=event.timestamp,
-                         displaymarking=displaymarking,
-                         message_id=message_id,
-                         msg_log_id=event.msg_log_id,
-                         additional_data=event.additional_data)
+        self._scrolled_view.acknowledge_message(event)
 
     def _on_message_received(self, event: events.MessageReceived) -> None:
         if not self._is_event_processable(event):
             return
 
-        if self.is_groupchat:
-            return
-
-        if not event.msgtxt:
-            return
-
-        kind = 'incoming'
-        if event.properties.is_sent_carbon:
-            kind = 'outgoing'
-
-        self.add_message(event.msgtxt,
-                         kind,
-                         tim=event.properties.timestamp,
-                         displaymarking=event.displaymarking,
-                         msg_log_id=event.msg_log_id,
-                         message_id=event.properties.id,
-                         stanza_id=event.stanza_id,
-                         additional_data=event.additional_data)
-
-    def _on_mam_message_received(self,
-                                 event: events.MamMessageReceived) -> None:
+        self._add_message(event.message)
 
+    def _on_message_corrected(self, event: events.MessageCorrected) -> None:
         if not self._is_event_processable(event):
             return
 
-        if isinstance(self.contact, GroupchatContact):
-
-            if not event.properties.type.is_groupchat:
-                return
-            if event.archive_jid != self.contact.jid:
-                return
-            self.add_muc_message(event.msgtxt,
-                                 tim=event.properties.mam.timestamp,
-                                 contact=event.properties.muc_nickname,
-                                 message_id=event.properties.id,
-                                 stanza_id=event.stanza_id,
-                                 additional_data=event.additional_data)
-
-        else:
-
-            if event.properties.is_muc_pm:
-                if not event.properties.jid == self.contact.jid:
-                    return
-            else:
-                if not event.properties.jid.bare_match(self.contact.jid):
-                    return
-
-            kind = 'incoming'
-            if event.kind == KindConstant.CHAT_MSG_SENT:
-                kind = 'outgoing'
-
-            self.add_message(event.msgtxt,
-                             kind,
-                             tim=event.properties.mam.timestamp,
-                             message_id=event.properties.id,
-                             stanza_id=event.stanza_id,
-                             additional_data=event.additional_data)
-
-    def _on_gc_message_received(self, event: events.GcMessageReceived) -> None:
-        if not self._is_event_processable(event):
-            return
-
-        if event.properties.muc_nickname is None:
-            # message from server
-            self.add_muc_message(event.msgtxt,
-                                 tim=event.properties.timestamp,
-                                 displaymarking=event.displaymarking,
-                                 additional_data=event.additional_data)
-        else:
-            self.add_muc_message(event.msgtxt,
-                                 tim=event.properties.timestamp,
-                                 contact=event.properties.muc_nickname,
-                                 displaymarking=event.displaymarking,
-                                 message_id=event.properties.id,
-                                 stanza_id=event.stanza_id,
-                                 msg_log_id=event.msg_log_id,
-                                 additional_data=event.additional_data)
-
-    def _on_message_updated(self, event: events.MessageUpdated) -> None:
-        if not self._is_event_processable(event):
-            return
-
-        self._scrolled_view.correct_message(
-            event.correct_id, event.msgtxt, event.nickname)
+        self._scrolled_view.correct_message(event)
 
     def _on_message_moderated(self, event: events.MessageModerated) -> None:
         if not self._is_event_processable(event):
             return
 
-        text = get_retraction_text(
-            self.contact.account,
-            event.moderation.moderator_jid,
+        text = get_moderation_text(
+            event.moderation.by,
             event.moderation.reason)
-        self._scrolled_view.show_message_retraction(
+        self._scrolled_view.show_message_moderation(
             event.moderation.stanza_id, text)
 
     def _on_receipt_received(self, event: events.ReceiptReceived) -> None:
         if not self._is_event_processable(event):
             return
 
-        self._scrolled_view.show_receipt(event.receipt_id)
+        self._scrolled_view.set_receipt(event.receipt_id)
 
     def _on_displayed_received(self, event: events.DisplayedReceived) -> None:
         if not self._is_event_processable(event):
@@ -402,6 +384,12 @@ class ChatControl(EventHelper):
 
         self._scrolled_view.set_read_marker(event.marker_id)
 
+    def _on_reaction_updated(self, event: events.ReactionUpdated) -> None:
+        if not self._is_event_processable(event):
+            return
+
+        self._scrolled_view.update_message_reactions(event.reaction_id)
+
     def _on_message_error(self, event: events.MessageError) -> None:
         if not self._is_event_processable(event):
             return
@@ -428,38 +416,30 @@ class ChatControl(EventHelper):
         active_jid = app.call_manager.get_active_call_jid()
         # Don't add a second row if contact upgrades to video
         if active_jid is None:
-            self.add_call_message(event=event)
+            self._add_call_message(event=event)
 
-    def _on_file_request_event(self,
-                               event: Union[events.FileRequestReceivedEvent,
-                                            events.FileRequestSent]
-                               ) -> None:
+    def _on_file_request_event(
+        self,
+        event: events.FileRequestReceivedEvent | events.FileRequestSent
+    ) -> None:
 
         if not self._is_event_processable(event):
             return
 
-        self.add_jingle_file_transfer(event=event)
+        self._add_jingle_file_transfer(event=event)
 
     def _on_http_upload_started(self, event: events.HTTPUploadStarted) -> None:
-        self.add_file_transfer(event.transfer)
+        self._add_file_transfer(event.transfer)
 
     def _on_http_upload_error(self, event: events.HTTPUploadError) -> None:
         self.add_info_message(event.error_msg)
 
-    @property
-    def is_chat(self) -> bool:
-        return isinstance(self.contact, BareContact)
+    def _on_encryption_info(self, event: events.EncryptionInfo) -> None:
+        if not self._is_event_processable(event):
+            return
 
-    @property
-    def is_privatechat(self) -> bool:
-        return isinstance(self.contact, GroupchatParticipant)
-
-    @property
-    def is_groupchat(self) -> bool:
-        return isinstance(self.contact, GroupchatContact)
-
-    def mark_as_read(self) -> None:
-        self._jump_to_end_button.reset_unread_count()
+        if self._allow_add_message():
+            self._scrolled_view.add_encryption_info(event)
 
     def _on_autoscroll_changed(self,
                                _widget: ConversationView,
@@ -479,8 +459,8 @@ class ChatControl(EventHelper):
                                        param: GLib.Variant
                                        ) -> None:
 
-        log_line_id = param.get_uint32()
-        self._scrolled_view.enable_row_selection(log_line_id)
+        pk = param.get_uint32()
+        self._scrolled_view.enable_row_selection(pk)
         self._message_selection.set_no_show_all(False)
         self._message_selection.show_all()
 
@@ -490,17 +470,24 @@ class ChatControl(EventHelper):
     def _on_cancel_selection(self, _widget: MessageSelection) -> None:
         self._scrolled_view.disable_row_selection()
 
+    def _on_jump_to_message(self,
+                            _action: Gio.SimpleAction,
+                            param: GLib.Variant
+                            ) -> None:
+
+        pk, timestamp = param.unpack()
+        self.scroll_to_message(
+            pk,
+            dt.datetime.fromtimestamp(timestamp, dt.timezone.utc)
+        )
+
     def _on_jump_to_end(self, _button: Gtk.Button) -> None:
         self.reset_view()
 
-    def drag_data_file_transfer(self, selection: Gtk.SelectionData) -> None:
-        app.window.activate_action('send-file',
-                                   GLib.Variant('as', selection.get_uris()))
-
-    def get_our_nick(self) -> str:
+    def _get_our_nick(self) -> str:
         if isinstance(self.contact, GroupchatParticipant):
             muc_data = self.client.get_module('MUC').get_muc_data(
-                self.contact.jid.bare)
+                self.contact.jid.new_as_bare())
             if muc_data is not None:
                 return muc_data.nick
 
@@ -509,99 +496,59 @@ class ChatControl(EventHelper):
     def _allow_add_message(self) -> bool:
         return self._scrolled_view.get_lower_complete()
 
-    def add_command_output(self, text: str, is_error: bool) -> None:
-        self._scrolled_view.add_command_output(text, is_error)
-
-    def add_info_message(self,
-                         text: str,
-                         timestamp: Optional[float] = None
-                         ) -> None:
-
-        self._scrolled_view.add_info_message(text, timestamp)
-
-    def add_file_transfer(self, transfer: HTTPFileTransfer) -> None:
+    def _add_file_transfer(self, transfer: HTTPFileTransfer) -> None:
         self._scrolled_view.add_file_transfer(transfer)
 
-    def add_jingle_file_transfer(self,
-                                 event: Union[
-                                     events.FileRequestReceivedEvent,
-                                     events.FileRequestSent,
-                                     None]
-                                 ) -> None:
+    def _add_jingle_file_transfer(
+        self,
+        event: events.FileRequestReceivedEvent | events.FileRequestSent | None
+    ) -> None:
         if self._allow_add_message():
             self._scrolled_view.add_jingle_file_transfer(event)
 
-    def add_call_message(self, event: events.JingleRequestReceived) -> None:
+    def _add_call_message(self, event: events.JingleRequestReceived) -> None:
         if self._allow_add_message():
             self._scrolled_view.add_call_message(event=event)
 
-    def _add_message(self,
-                     text: str,
-                     kind: str,
-                     name: str,
-                     tim: float,
-                     displaymarking: Optional[Displaymarking] = None,
-                     msg_log_id: Optional[int] = None,
-                     message_id: Optional[str] = None,
-                     stanza_id: Optional[str] = None,
-                     additional_data: Optional[AdditionalDataDict] = None
-                     ) -> None:
-
-        if additional_data is None:
-            additional_data = AdditionalDataDict()
-
+    def _add_message(self, message: Message) -> None:
+        # TODO: Unify with _add_db_row()
         if self._allow_add_message():
-            self._scrolled_view.add_message(
-                text,
-                kind,
-                name,
-                tim,
-                display_marking=displaymarking,
-                message_id=message_id,
-                stanza_id=stanza_id,
-                log_line_id=msg_log_id,
-                additional_data=additional_data)
+            self._scrolled_view.add_message_from_db(message)
 
             if not self._scrolled_view.get_autoscroll():
-                if kind == 'outgoing':
+                if message.direction == ChatDirection.OUTGOING:
                     self._scrolled_view.scroll_to_end()
                 else:
                     self._jump_to_end_button.add_unread_count()
         else:
             self._jump_to_end_button.add_unread_count()
 
-    def reset_view(self) -> None:
-        self._scrolled_view.reset()
+    def _add_db_row(self, message: Message):
+        if message.filetransfers:
+            self._scrolled_view.add_jingle_file_transfer(message=message)
+            return
 
-    def get_autoscroll(self) -> bool:
-        return self._scrolled_view.get_autoscroll()
+        if message.call is not None:
+            self._scrolled_view.add_call_message(message=message)
+            return
 
-    def scroll_to_message(self, log_line_id: int, timestamp: float) -> None:
-        row = self._scrolled_view.get_row_by_log_line_id(log_line_id)
-        if row is None:
-            # Clear view and reload conversation around timestamp
-            self._scrolled_view.block_signals(True)
-            self.reset_view()
-            before, at_after = app.storage.archive.get_conversation_around(
-                self.contact.account, self.contact.jid, timestamp)
-            self.add_messages(before)
-            self.add_messages(at_after)
+        self._scrolled_view.add_message_from_db(message)
 
-        GLib.idle_add(
-            self._scrolled_view.scroll_to_message_and_highlight,
-            log_line_id)
-        GLib.idle_add(self._scrolled_view.block_signals, False)
+    def _add_messages(self, messages: list[Message]):
+        for msg in messages:
+            self._add_db_row(msg)
 
-    def _request_messages(self, before: bool) -> list[ConversationRow]:
+    def _request_messages(self, before: bool) -> Sequence[Message]:
         if before:
-            row = self._scrolled_view.get_first_message_row()
+            row = self._scrolled_view.get_first_row()
         else:
-            row = self._scrolled_view.get_last_message_row()
+            row = self._scrolled_view.get_last_row()
 
         if row is None:
-            timestamp = time.time()
+            timestamp = dt.datetime.now(dt.timezone.utc)
         else:
-            timestamp = row.db_timestamp
+            timestamp = dt.datetime.fromtimestamp(
+                row.db_timestamp, dt.timezone.utc)
 
         return app.storage.archive.get_conversation_before_after(
             self.contact.account,
@@ -628,7 +575,7 @@ class ChatControl(EventHelper):
                                        REQUEST_LINES_COUNT)
 
     def _request_history(self,
-                         _scrolled: Gtk.ScrolledWindow,
+                         _widget: Any,
                          before: bool
                          ) -> None:
 
@@ -641,7 +588,7 @@ class ChatControl(EventHelper):
         assert self._contact is not None
         for row in rows:
             if not isinstance(row, events.ApplicationEvent):
-                self.add_messages([row])
+                self._add_messages([row])
 
             elif isinstance(row, events.MUCUserJoined):
                 self._process_muc_user_joined(row)
@@ -658,6 +605,9 @@ class ChatControl(EventHelper):
             elif isinstance(row, events.MUCUserAffiliationChanged):
                 self._process_muc_user_affiliation_changed(row)
 
+            elif isinstance(row, events.MUCAffiliationChanged):
+                self._process_room_affiliation_changed(row)
+
             elif isinstance(row, events.MUCUserRoleChanged):
                 self._process_muc_user_role_changed(row)
 
@@ -685,102 +635,20 @@ class ChatControl(EventHelper):
         self._scrolled_view.block_signals(False)
 
     @staticmethod
-    def _sort_request_rows(messages: list[ConversationRow],
+    def _sort_request_rows(messages: Sequence[Message],
                            event_rows: list[events.ApplicationEvent],
                            before: bool
                            ) -> list[HistoryRowT]:
 
         def sort_func(obj: HistoryRowT) -> float:
-            if isinstance(obj, events.ApplicationEvent):
-                return obj.timestamp  # pyright: ignore
-            return obj.time
+            return obj.timestamp  # pyright: ignore
+
+        assert isinstance(messages, list)
 
         rows = messages + event_rows
         rows.sort(key=sort_func, reverse=before)
         return rows
 
-    def add_messages(self, messages: list[ConversationRow]):
-        for msg in messages:
-            if msg.kind in (KindConstant.FILE_TRANSFER_INCOMING,
-                            KindConstant.FILE_TRANSFER_OUTGOING):
-                assert msg.additional_data is not None
-                if msg.additional_data.get_value('gajim', 'type') == 'jingle':
-                    self._scrolled_view.add_jingle_file_transfer(
-                        db_message=msg)
-                continue
-
-            if msg.kind in (KindConstant.CALL_INCOMING,
-                            KindConstant.CALL_OUTGOING):
-                self._scrolled_view.add_call_message(db_message=msg)
-                continue
-
-            if not msg.message:
-                continue
-
-            message_text = msg.message
-
-            contact_name = msg.contact_name
-            kind = 'incoming'
-            if msg.kind in (
-                    KindConstant.SINGLE_MSG_RECV, KindConstant.CHAT_MSG_RECV):
-                kind = 'incoming'
-                contact_name = self.contact.name
-            elif msg.kind == KindConstant.GC_MSG:
-                kind = 'incoming'
-            elif msg.kind in (
-                    KindConstant.SINGLE_MSG_SENT, KindConstant.CHAT_MSG_SENT):
-                kind = 'outgoing'
-                contact_name = self.get_our_nick()
-            else:
-                log.warning('kind attribute could not be processed'
-                            'while adding message')
-
-            if msg.additional_data is not None:
-                retracted_by = msg.additional_data.get_value('retracted', 'by')
-                if retracted_by is not None:
-                    reason = msg.additional_data.get_value(
-                        'retracted', 'reason')
-                    message_text = get_retraction_text(
-                        self.contact.account, retracted_by, reason)
-
-            self._scrolled_view.add_message(
-                message_text,
-                kind,
-                contact_name,
-                msg.time,
-                additional_data=msg.additional_data,
-                message_id=msg.message_id,
-                stanza_id=msg.stanza_id,
-                log_line_id=msg.log_line_id,
-                marker=msg.marker,
-                error=msg.error)
-
-    def add_message(self,
-                    text: str,
-                    kind: str,
-                    tim: float,
-                    displaymarking: Optional[Displaymarking] = None,
-                    msg_log_id: Optional[int] = None,
-                    stanza_id: Optional[str] = None,
-                    message_id: Optional[str] = None,
-                    additional_data: Optional[AdditionalDataDict] = None
-                    ) -> None:
-
-        if kind == 'incoming':
-            name = self.contact.name
-        else:
-            name = self.get_our_nick()
-
-        self._add_message(text,
-                          kind,
-                          name,
-                          tim,
-                          displaymarking=displaymarking,
-                          msg_log_id=msg_log_id,
-                          message_id=message_id,
-                          stanza_id=stanza_id,
-                          additional_data=additional_data)
-
     def _on_user_nickname_changed(self,
                                   _contact: types.GroupchatContact,
                                   _signal_name: str,
@@ -895,8 +763,7 @@ class ChatControl(EventHelper):
                             actor=actor,
                             reason=reason)
         else:
-            message = _('** Affiliation of {nick} has been set to '
-                        '{affiliation}{actor}{reason}').format(
+            message = self.__format_affiliation_change(
                             nick=event.nick,
                             affiliation=affiliation,
                             actor=actor,
@@ -904,6 +771,36 @@ class ChatControl(EventHelper):
 
         self.add_info_message(message, event.timestamp)
 
+    def _on_room_affiliation_changed(
+        self,
+        _contact: GroupchatContact,
+        _signal_name: str,
+        event: events.MUCAffiliationChanged,
+    ) -> None:
+        self._process_room_affiliation_changed(event)
+
+    def _process_room_affiliation_changed(
+        self,
+        event: events.MUCAffiliationChanged,
+    ) -> None:
+        self.add_info_message(
+            self.__format_affiliation_change(
+                event.nick,
+                helpers.get_uf_affiliation(event.affiliation)),
+            event.timestamp)
+
+    @staticmethod
+    def __format_affiliation_change(nick: str,
+                                    affiliation: str,
+                                    actor: str = '',
+                                    reason: str = '') -> str:
+        return _('** Affiliation of {nick} has been set to '
+                 '{affiliation}{actor}{reason}').format(
+            nick=nick,
+            affiliation=affiliation,
+            actor=actor,
+            reason=reason)
+
     def _on_user_role_changed(self,
                               _contact: GroupchatContact,
                               _signal_name: str,
@@ -1047,7 +944,8 @@ class ChatControl(EventHelper):
                                           event: events.MUCRoomConfigFinished
                                           ) -> None:
 
-        self.add_info_message(_('A new group chat has been created'))
+        self.add_info_message(_('A new group chat has been created'),
+                              event.timestamp)
 
     def _on_room_presence_error(self,
                                 _contact: GroupchatContact,
@@ -1061,8 +959,7 @@ class ChatControl(EventHelper):
                                          event: events.MUCRoomPresenceError
                                          ) -> None:
 
-        error_message = to_user_string(event.error)
-        self.add_info_message(_('Error: %s') % error_message)
+        self.add_info_message(_('Error: %s') % event.error, event.timestamp)
 
     def _on_room_destroyed(self,
                            _contact: GroupchatContact,
@@ -1082,8 +979,8 @@ class ChatControl(EventHelper):
         message = _('Group chat has been destroyed%s') % reason
 
         if event.alternate is not None:
-            message += '\n' + _('You can join this group chat instead: '
-                                'xmpp:%s?join') % str(event.alternate)
+            message += '\n' + _('You can join this group chat instead: %s') % (
+                event.alternate.to_iri(XmppUriQuery.JOIN.value))
 
         self.add_info_message(message, event.timestamp)
 
@@ -1183,35 +1080,6 @@ class ChatControl(EventHelper):
 
         self.add_info_message(message, event.timestamp)
 
-    def add_muc_message(self,
-                        text: str,
-                        tim: float,
-                        contact: str = '',
-                        displaymarking: Optional[Displaymarking] = None,
-                        message_id: Optional[str] = None,
-                        stanza_id: Optional[str] = None,
-                        msg_log_id: Optional[int] = None,
-                        additional_data: Optional[AdditionalDataDict] = None,
-                        ) -> None:
-
-        assert isinstance(self._contact, GroupchatContact)
-
-        if contact == self._contact.nickname:
-            kind = 'outgoing'
-        else:
-            kind = 'incoming'
-            # muc-specific chatstate
-
-        self._add_message(text,
-                          kind,
-                          contact,
-                          tim,
-                          displaymarking=displaymarking,
-                          message_id=message_id,
-                          stanza_id=stanza_id,
-                          msg_log_id=msg_log_id,
-                          additional_data=additional_data)
-
     def _on_room_subject(self,
                          contact: GroupchatContact,
                          _signal_name: str,
