@@ -1,29 +1,20 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
 import logging
 
+import emoji
 from gi.repository import Gdk
 from gi.repository import GLib
 from gi.repository import Gtk
 from gi.repository import Pango
 
+from gajim.common.const import URIType
 from gajim.common.helpers import open_uri
 from gajim.common.helpers import parse_uri
-from gajim.common.structs import URIType
 from gajim.common.styling import BaseHyperlink
 from gajim.common.styling import PlainBlock
 from gajim.common.styling import process_uris
@@ -66,7 +57,9 @@ class MessageLabel(Gtk.Label):
         self.set_selectable(selectable)
         self.set_line_wrap(True)
         self.set_xalign(0)
-        self.set_line_wrap_mode(Pango.WrapMode.WORD_CHAR)
+        # WrapMode.WORD_CHAR can cause a segfault
+        # https://gitlab.gnome.org/GNOME/pango/-/issues/798
+        self.set_line_wrap_mode(Pango.WrapMode.WORD)
         self.set_track_visited_links(False)
 
         self._account = account
@@ -107,13 +100,23 @@ class MessageLabel(Gtk.Label):
     def print_text_with_styling(self, block: PlainBlock) -> None:
         text = self._build_link_markup(block.text, block.uris)
         self.set_markup(text)
-
         if len(self.get_text()) > MAX_MESSAGE_LENGTH:
             # Limit message styling processing
             return
 
         self.set_attributes(make_pango_attributes(block))
 
+        stripped = text.strip()
+        # https://github.com/carpedm20/emoji/issues/300
+        # purely_emoji() return True for the empty string
+        if stripped and emoji.purely_emoji(stripped):
+            emoji_count = emoji.emoji_count(stripped)
+            if emoji_count == 1:
+                classname = 'gajim-single-emoji-msg'
+            else:
+                classname = 'gajim-emoji-msg'
+            self.get_style_context().add_class(classname)
+
     def add_action_phrase(self, text: str, nickname: str) -> None:
         text = text.replace('/me', f'* {nickname}', 1)
         uris = process_uris(text)
