@@ -8,17 +8,7 @@
 #
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 # The appearance of the treeview, and parts of the dialog, are controlled by
 # AgentBrowser (sub-)classes. Methods that probably should be overridden when
@@ -58,11 +48,11 @@ from gajim.common import helpers
 from gajim.common.const import StyleAttr
 from gajim.common.i18n import _
 
-from .builder import get_builder
-from .component_search import ComponentSearch
-from .dialogs import ErrorDialog
-from .util import icon_exists
-from .util import open_window
+from gajim.gtk.builder import get_builder
+from gajim.gtk.component_search import ComponentSearch
+from gajim.gtk.dialogs import ErrorDialog
+from gajim.gtk.util import icon_exists
+from gajim.gtk.util import open_window
 
 LABELS = {
     1: _('This service has not yet responded with detailed information'),
@@ -208,7 +198,7 @@ def get_agent_address(jid, node=None):
     Get an agent's address for displaying in the GUI
     '''
     if node:
-        return '%s@%s' % (node, str(jid))
+        return f'{node}@{jid}'
     return str(jid)
 
 
@@ -301,7 +291,7 @@ class ServicesCache:
             service_name = addr.split('.')[0]
 
         # Or load it
-        icon_name = 'gajim-agent-%s' % service_name
+        icon_name = f'gajim-agent-{service_name}'
         if icon_exists(icon_name):
             return icon_name
         return 'gajim-agent-jabber'
@@ -416,7 +406,7 @@ class ServicesCache:
                                         type='im',
                                         name=result.node)]
 
-        self._on_agent_info(str(result.jid), result.node, result.identities,
+        self._on_agent_info(str(result.jid), result.node, identities,
                             result.features, result.dataforms)
 
     def _disco_info_error(self, result):
@@ -506,7 +496,7 @@ class ServiceDiscoveryWindow:
         self._account = account
         self.parent = parent
         if not jid:
-            jid = app.settings.get_account_setting(account, 'hostname')
+            jid = app.get_hostname_from_account(account)
             node = None
 
         self.jid = None
@@ -1139,7 +1129,7 @@ class ToplevelAgentBrowser(AgentBrowser):
 
     def add_self_line(self):
         addr = get_agent_address(self.jid, self.node)
-        descr = '<b>%s</b>' % addr
+        descr = f'<b>{addr}</b>'
         # Guess which kind of service this is
         identities = []
         type_ = app.get_transport_name_from_jid(self.jid,
@@ -1331,7 +1321,7 @@ class ToplevelAgentBrowser(AgentBrowser):
         if not iter_:
             return
         service = model[iter_][0]
-        open_window('AdHocCommands', account=self.account, jid=service)
+        open_window('AdHocCommands', account=self.account, jids=[service])
 
     def _on_register_button_clicked(self, widget=None):
         '''
@@ -1541,9 +1531,9 @@ class ToplevelAgentBrowser(AgentBrowser):
         # Row text
         addr = get_agent_address(jid, node)
         if 'name' in item:
-            descr = '<b>%s</b>\n%s' % (item['name'], addr)
+            descr = f'<b>{item["name"]}</b>\n{addr}'
         else:
-            descr = '<b>%s</b>' % addr
+            descr = f'<b>{addr}</b>'
         # Guess which kind of service this is
         identities = []
         type_ = app.get_transport_name_from_jid(
@@ -1570,21 +1560,21 @@ class ToplevelAgentBrowser(AgentBrowser):
     def _update_item(self, iter_, jid, node, item):
         addr = get_agent_address(jid, node)
         if 'name' in item:
-            descr = '<b>%s</b>\n%s' % (item['name'], addr)
+            descr = f'<b>{item["name"]}</b>\n{addr}'
         else:
-            descr = '<b>%s</b>' % addr
+            descr = f'<b>{addr}</b>'
         self.model[iter_][3] = descr
 
     def _update_info(self, iter_, jid, node, identities, features, data):
         addr = get_agent_address(jid, node)
         if not identities:
-            descr = '<b>%s</b>' % addr
+            descr = '<b>{addr}</b>'
         else:
             name = identities[0].name or ''
             if name:
-                descr = '<b>%s</b>\n%s' % (name, addr)
+                descr = f'<b>{name}</b>\n{addr}'
             else:
-                descr = '<b>%s</b>' % addr
+                descr = f'<b>{addr}</b>'
 
         # Update progress
         self._progress += 1
