@@ -1,23 +1,13 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
-from typing import Optional
+from __future__ import annotations
 
+import logging
 import time
 
 from gi.repository import Gdk
-from gi.repository import GLib
 from gi.repository import Gtk
 from nbxmpp import JID
 from nbxmpp.namespaces import Namespace
@@ -27,13 +17,17 @@ from nbxmpp.structs import MucSubject
 from gajim.common import app
 from gajim.common.const import AvatarSize
 from gajim.common.const import RFC5646_LANGUAGE_TAGS
+from gajim.common.const import XmppUriQuery
 from gajim.common.helpers import get_groupchat_name
 from gajim.common.helpers import open_uri
 from gajim.common.i18n import _
 from gajim.common.i18n import p_
+from gajim.common.modules.contacts import GroupchatContact
 
-from .builder import get_builder
-from .util import make_href_markup
+from gajim.gtk.builder import get_builder
+from gajim.gtk.util import make_href_markup
+
+log = logging.getLogger('gajim.gtk.groupchat_info')
 
 MUC_FEATURES = {
     'muc_open': (
@@ -46,12 +40,12 @@ MUC_FEATURES = {
         _('This group chat is restricted '
           'to members only')),
     'muc_nonanonymous': (
-        'feather-shield-off-symbolic',
+        'feather-eye-symbolic',
         p_('Group chat feature', 'Not Anonymous'),
         _('All other group chat participants '
           'can see your XMPP address')),
     'muc_semianonymous': (
-        'feather-shield-symbolic',
+        'feather-eye-off-symbolic',
         p_('Group chat feature', 'Semi-Anonymous'),
         _('Only moderators can see your XMPP address')),
     'muc_moderated': (
@@ -65,11 +59,11 @@ MUC_FEATURES = {
         _('Participants entering this group chat are '
           'allowed to send messages')),
     'muc_public': (
-        'feather-eye-symbolic',
+        'lucide-megaphone-symbolic',
         p_('Group chat feature', 'Public'),
         _('Group chat can be found via search')),
     'muc_hidden': (
-        'feather-eye-off-symbolic',
+        'lucide-megaphone-off-symbolic',
         p_('Group chat feature', 'Hidden'),
         _('This group chat can not be found via search')),
     'muc_passwordprotected': (
@@ -101,7 +95,7 @@ MUC_FEATURES = {
 
 class GroupChatInfoScrolled(Gtk.ScrolledWindow):
     def __init__(self,
-                 account: Optional[str] = None,
+                 account: str | None = None,
                  width: int = 300,
                  minimal: bool = False
                  ) -> None:
@@ -122,23 +116,26 @@ class GroupChatInfoScrolled(Gtk.ScrolledWindow):
                             Gtk.PolicyType.AUTOMATIC)
 
         self._account = account
-        self._info: Optional[DiscoInfo] = None
+        self._contact: GroupchatContact | None = None
+        self._info: DiscoInfo | None = None
 
         self._ui = get_builder('groupchat_info_scrolled.ui')
         self.add(self._ui.info_grid)
         self._ui.connect_signals(self)
         self.show_all()
 
-    def get_account(self) -> Optional[str]:
+    def get_account(self) -> str | None:
         return self._account
 
     def set_account(self, account: str) -> None:
         self._account = account
 
-    def get_jid(self) -> Optional[JID]:
+    def get_jid(self) -> JID | None:
+        if self._contact is not None:
+            return self._contact.jid
         return self._info.jid
 
-    def set_subject(self, muc_subject: Optional[MucSubject]) -> None:
+    def set_subject(self, muc_subject: MucSubject | None) -> None:
         if muc_subject is None:
             return
 
@@ -153,11 +150,21 @@ class GroupChatInfoScrolled(Gtk.ScrolledWindow):
         self._ui.author_label.set_visible(has_author)
 
         has_subject = bool(muc_subject.text)
-        subject = GLib.markup_escape_text(muc_subject.text)
-        self._ui.subject.set_markup(make_href_markup(subject))
+        self._ui.subject.set_markup(make_href_markup(muc_subject.text))
         self._ui.subject.set_visible(has_subject)
         self._ui.subject_label.set_visible(has_subject)
 
+    def set_info_from_contact(self, contact: GroupchatContact) -> None:
+        self._contact = contact
+        disco_info = contact.get_disco()
+        if disco_info is not None:
+            self.set_from_disco_info(disco_info)
+            return
+
+        self._ui.address.set_text(str(contact.jid))
+        self._ui.address.set_tooltip_text(str(contact.jid))
+        self._ui.address_copy_button.set_sensitive(True)
+
     def set_from_disco_info(self, info: DiscoInfo) -> None:
         self._info = info
         # Set name
@@ -178,13 +185,14 @@ class GroupChatInfoScrolled(Gtk.ScrolledWindow):
 
         # Set description
         has_desc = bool(info.muc_description)
-        self._ui.description.set_text(info.muc_description or '')
+        self._ui.description.set_markup(make_href_markup(info.muc_description))
         self._ui.description.set_visible(has_desc)
         self._ui.description_label.set_visible(has_desc)
 
         # Set address
         self._ui.address.set_text(str(info.jid))
         self._ui.address.set_tooltip_text(str(info.jid))
+        self._ui.address_copy_button.set_sensitive(True)
 
         if self._minimal:
             return
@@ -200,7 +208,12 @@ class GroupChatInfoScrolled(Gtk.ScrolledWindow):
         has_contacts = bool(info.muc_contacts)
         if has_contacts:
             for contact in info.muc_contacts:
-                self._ui.contact_box.add(self._get_contact_button(contact))
+                try:
+                    jid = JID.from_string(contact).new_as_bare()
+                except Exception as e:
+                    log.debug('Bad MUC contact address %s: %s', contact, str(e))
+                else:
+                    self._ui.contact_box.add(self._get_contact_button(jid))
 
         self._ui.contact_box.set_visible(has_contacts)
         self._ui.contact_label.set_visible(has_contacts)
@@ -249,8 +262,12 @@ class GroupChatInfoScrolled(Gtk.ScrolledWindow):
         grid.show_all()
 
     def _on_copy_address(self, _button: Gtk.Button) -> None:
+        if self._contact is not None:
+            jid = self._contact.jid
+        else:
+            jid = self._info.jid
         clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
-        clipboard.set_text(f'xmpp:{self._info.jid}?join', -1)
+        clipboard.set_text(jid.to_iri(XmppUriQuery.JOIN.value), -1)
 
     @staticmethod
     def _on_activate_log_link(button: Gtk.LinkButton) -> int:
@@ -258,7 +275,7 @@ class GroupChatInfoScrolled(Gtk.ScrolledWindow):
         return Gdk.EVENT_STOP
 
     def _on_activate_contact_link(self, button: Gtk.LinkButton) -> int:
-        open_uri(f'xmpp:{button.get_uri()}?message', account=self._account)
+        open_uri(button.get_uri(), account=self._account)
         return Gdk.EVENT_STOP
 
     @staticmethod
@@ -283,8 +300,9 @@ class GroupChatInfoScrolled(Gtk.ScrolledWindow):
         label.set_valign(Gtk.Align.START)
         return label
 
-    def _get_contact_button(self, contact: str) -> Gtk.Button:
-        button = Gtk.LinkButton(label=contact)
+    def _get_contact_button(self, contact: JID) -> Gtk.Button:
+        button = Gtk.LinkButton(contact.to_iri(XmppUriQuery.MESSAGE.value),
+                                label=str(contact))
         button.set_halign(Gtk.Align.START)
         button.get_style_context().add_class('link-button')
         button.connect('activate-link', self._on_activate_contact_link)
