@@ -1,21 +1,10 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
 from typing import Any
-from typing import Optional
 
 import locale
 import logging
@@ -36,15 +25,16 @@ from gajim.common.events import MUCNicknameChanged
 from gajim.common.helpers import get_uf_affiliation
 from gajim.common.helpers import get_uf_role
 from gajim.common.helpers import jid_is_blocked
+from gajim.common.i18n import p_
 from gajim.common.modules.contacts import GroupchatContact
 
-from .builder import get_builder
-from .menus import get_groupchat_participant_menu
-from .tooltips import GCTooltip
-from .util import EventHelper
-from .util import GajimPopover
+from gajim.gtk.builder import get_builder
+from gajim.gtk.menus import get_groupchat_participant_menu
+from gajim.gtk.tooltips import GCTooltip
+from gajim.gtk.util import EventHelper
+from gajim.gtk.util import GajimPopover
 
-log = logging.getLogger('gajim.gui.groupchat_roster')
+log = logging.getLogger('gajim.gtk.groupchat_roster')
 
 
 AffiliationRoleSortOrder = {
@@ -84,7 +74,7 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         self._tooltip = GCTooltip()
 
         self._ui = get_builder('groupchat_roster.ui')
-        self.add(self._ui.scrolled)
+        self.add(self._ui.box)
 
         self._contact_refs: dict[str, Gtk.TreeRowReference] = {}
         self._group_refs: dict[str, Gtk.TreeRowReference] = {}
@@ -93,7 +83,10 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         self._store.set_sort_func(Column.TEXT, self._tree_compare_iters)
 
         self._roster = self._ui.roster_treeview
-        self._roster.set_search_equal_func(self._search_func)
+
+        self._filter_string = ''
+        self._modelfilter = self._store.filter_new()
+        self._modelfilter.set_visible_func(self._visible_func)
         self._roster.set_has_tooltip(True)
 
         self._ui.contact_column.set_fixed_width(
@@ -103,9 +96,6 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
 
         self._ui.connect_signals(self)
 
-        app.settings.connect_signal(
-            'hide_groupchat_occupants_list', self._hide_roster)
-
         self.register_events([
             ('theme-update', ged.GUI2, self._on_theme_update),
         ])
@@ -126,6 +116,9 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         self.set_visible(not hide_roster)
 
     def _on_reveal(self, revealer: Gtk.Revealer, param: Any) -> None:
+        if self._contact is None:
+            return
+
         if revealer.get_reveal_child():
             self._load_roster()
         else:
@@ -137,12 +130,12 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
 
         log.info('Clear')
         self._unload_roster()
+        app.settings.disconnect_signals(self)
         self._contact.disconnect_signal(self, 'state-changed')
         self._contact = None
 
     def switch_contact(self, contact: types.ChatContactT) -> None:
-        if self._contact is not None:
-            self.clear()
+        self.clear()
 
         is_groupchat = isinstance(contact, GroupchatContact)
         hide_roster = app.settings.get('hide_groupchat_occupants_list')
@@ -153,6 +146,8 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         log.info('Switch to %s (%s)', contact.jid, contact.account)
 
         contact.connect('state-changed', self._on_muc_state_changed)
+        app.settings.connect_signal(
+            'hide_groupchat_occupants_list', self._hide_roster)
 
         self._contact = contact
 
@@ -181,6 +176,11 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
             self._tooltip.clear_tooltip()
             return False
 
+        path = self._modelfilter.convert_path_to_child_path(path)
+        if path is None:
+            self._tooltip.clear_tooltip()
+            return False
+
         iter_ = None
         try:
             iter_ = self._store.get_iter(path)
@@ -201,16 +201,26 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         tooltip.set_custom(widget)
         return value
 
-    @staticmethod
-    def _search_func(model: Gtk.TreeModel,
-                     _column: int,
-                     search_text: str,
-                     iter_: Gtk.TreeIter
-                     ) -> bool:
+    def _on_search_changed(self, widget: Gtk.SearchEntry) -> None:
+        self._filter_string = widget.get_text().lower()
+        self._modelfilter.refilter()
+        self._roster.expand_all()
 
-        return search_text.lower() not in model[iter_][1].lower()
+    def _visible_func(self,
+                      model: Gtk.TreeModelFilter,
+                      iter_: Gtk.TreeIter,
+                      *_data: Any
+                      ) -> bool:
 
-    def _get_group_iter(self, group_name: str) -> Optional[Gtk.TreeIter]:
+        if not self._filter_string:
+            return True
+
+        if not model[iter_][Column.IS_CONTACT]:
+            return True
+
+        return self._filter_string in model[iter_][Column.TEXT].lower()
+
+    def _get_group_iter(self, group_name: str) -> Gtk.TreeIter | None:
         try:
             ref = self._group_refs[group_name]
         except KeyError:
@@ -221,7 +231,7 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
             return None
         return self._store.get_iter(path)
 
-    def _get_contact_iter(self, nick: str) -> Optional[Gtk.TreeIter]:
+    def _get_contact_iter(self, nick: str) -> Gtk.TreeIter | None:
         try:
             ref = self._contact_refs[nick]
         except KeyError:
@@ -340,7 +350,7 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
                              renderer: Gtk.CellRenderer,
                              model: Gtk.TreeModel,
                              iter_: Gtk.TreeIter,
-                             _user_data: Optional[object]
+                             _user_data: object | None
                              ) -> None:
 
         has_parent = bool(model.iter_parent(iter_))
@@ -367,7 +377,10 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
                                  _column: Gtk.TreeViewColumn
                                  ) -> None:
 
-        iter_ = self._store.get_iter(path)
+        child_path = self._modelfilter.convert_path_to_child_path(path)
+        assert child_path is not None
+
+        iter_ = self._store.get_iter(child_path)
         if self._store.iter_parent(iter_) is None:
             # This is a group row
             return
@@ -400,7 +413,7 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
                                       event: Gdk.EventButton
                                       ) -> None:
 
-        if event.button not in (2, 3):
+        if event.button == Gdk.BUTTON_PRIMARY:
             return
 
         pos = treeview.get_path_at_pos(int(event.x), int(event.y))
@@ -411,6 +424,10 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         if path is None:
             return
 
+        path = self._modelfilter.convert_path_to_child_path(path)
+        if path is None:
+            return
+
         iter_ = self._store.get_iter(path)
         if self._store.iter_parent(iter_) is None:
             # Group row
@@ -422,10 +439,10 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         if self._contact.nickname == nick:
             return
 
-        if event.button == 3:  # right click
+        if event.button == Gdk.BUTTON_SECONDARY:
             self._show_contact_menu(nick, event)
 
-        # if event.button == 2:  # middle click
+        # if event.button == Gdk.BUTTON_MIDDLE:
             # self.roster.emit('row-activated', nick)
 
     def _show_contact_menu(self, nick: str, event: Gdk.EventButton) -> None:
@@ -437,7 +454,7 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
                                               self_contact,
                                               contact)
 
-        popover = GajimPopover(menu, relative_to=self, event=event)
+        popover = GajimPopover(menu, relative_to=self._roster, event=event)
         popover.popup()
 
     def _on_muc_state_changed(self,
@@ -455,7 +472,7 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
                             model: Gtk.TreeModel,
                             iter1: Gtk.TreeIter,
                             iter2: Gtk.TreeIter,
-                            _user_data: Optional[object]
+                            _user_data: object | None
                             ) -> int:
         '''
         Compare two iterators to sort them
@@ -466,10 +483,15 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
             nick1 = model[iter1][Column.NICK_OR_GROUP]
             nick2 = model[iter2][Column.NICK_OR_GROUP]
 
+            assert self._contact is not None
+            our_nick = self._contact.nickname
+            if our_nick in (nick1, nick2):
+                # Always show our nickname at the top
+                return -1 if our_nick == nick1 else 1
+
             if not app.settings.get('sort_by_show_in_muc'):
                 return locale.strcoll(nick1.lower(), nick2.lower())
 
-            assert self._contact is not None
             contact1 = self._contact.get_resource(nick1)
             contact2 = self._contact.get_resource(nick2)
 
@@ -485,7 +507,7 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         group2_index = AffiliationRoleSortOrder[group2]
         return -1 if group1_index < group2_index else 1
 
-    def enable_sort(self, enable: bool) -> None:
+    def _enable_sort(self, enable: bool) -> None:
         column = Gtk.TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID
         if enable:
             column = Column.TEXT
@@ -511,8 +533,10 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         for participant in self._contact.get_participants():
             self._add_contact(participant)
 
-        self.enable_sort(True)
-        self._roster.set_model(self._store)
+        self._enable_sort(True)
+        self._roster.set_model(self._modelfilter)
+
+        self._ui.search_entry.set_text('')
         self._roster.expand_all()
 
     def _unload_roster(self) -> None:
@@ -525,14 +549,14 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
 
         self._roster.set_model(None)
         self._store.clear()
-        self.enable_sort(False)
+        self._enable_sort(False)
 
         self._contact_refs = {}
         self._group_refs = {}
 
     def invalidate_sort(self) -> None:
-        self.enable_sort(False)
-        self.enable_sort(True)
+        self._enable_sort(False)
+        self._enable_sort(True)
 
     def _redraw(self) -> None:
         self._roster.set_model(None)
@@ -550,6 +574,9 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         self._draw_avatar(contact)
 
         name = GLib.markup_escape_text(contact.name)
+        self_contact = self._contact.get_self()
+        if self_contact is not None and self_contact.name == nick:
+            name = p_('own nickname in group chat', '%s (You)' % nick)
 
         # Strike name if blocked
         fjid = f'{self._contact.jid}/{nick}'
@@ -557,10 +584,7 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
             name = f'<span strikethrough="true">{name}</span>'
 
         # add status msg, if not empty, under contact name
-        status = contact.status
-        if status is not None:
-            status = status.strip()
-
+        status = contact.status.strip()
         if status and app.settings.get('show_status_msgs_in_roster'):
             # Display only first line
             status = status.split('\n', 1)[0]
@@ -620,4 +644,6 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         return count
 
     def _on_theme_update(self, _event: ApplicationEvent) -> None:
+        if self._contact is None:
+            return
         self._redraw()
