@@ -1,23 +1,13 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
 from typing import Any
 from typing import cast
-from typing import Optional
-from typing import Union
+
+from collections.abc import Callable
 
 from gi.repository import Gio
 from gi.repository import GLib
@@ -26,7 +16,9 @@ from gi.repository import Pango
 from nbxmpp import JID
 
 from gajim.common import app
+from gajim.common import types
 from gajim.common.const import AvatarSize
+from gajim.common.const import SimpleClientState
 from gajim.common.events import MucDecline
 from gajim.common.events import MucInvitation
 from gajim.common.events import Notification
@@ -34,9 +26,17 @@ from gajim.common.events import SubscribePresenceReceived
 from gajim.common.events import UnsubscribedPresenceReceived
 from gajim.common.helpers import get_groupchat_name
 from gajim.common.i18n import _
+from gajim.common.modules.contacts import BareContact
+from gajim.common.modules.contacts import GroupchatContact
+from gajim.common.modules.contacts import GroupchatParticipant
 
-from .menus import get_subscription_menu
-from .util import open_window
+from gajim.gtk.menus import get_subscription_menu
+from gajim.gtk.util import open_window
+
+NotificationActionListT = list[
+    tuple[str,
+          Callable[[Gio.SimpleAction, GLib.Variant], Any],
+          str | None]]
 
 
 class NotificationManager(Gtk.ListBox):
@@ -44,6 +44,8 @@ class NotificationManager(Gtk.ListBox):
         Gtk.ListBox.__init__(self)
         self._account = account
         self._client = app.get_client(account)
+        self._client.connect_signal(
+            'state-changed', self._on_client_state_changed)
 
         self.set_selection_mode(Gtk.SelectionMode.NONE)
         self.set_halign(Gtk.Align.CENTER)
@@ -63,17 +65,27 @@ class NotificationManager(Gtk.ListBox):
     def _on_destroy(self, *args: Any) -> None:
         self._remove_actions()
 
+    def _on_client_state_changed(self,
+                                 _client: types.Client,
+                                 _signal_name: str,
+                                 _state: SimpleClientState
+                                 ) -> None:
+        self.update_actions()
+
     def _add_actions(self) -> None:
-        actions = [
+        actions: NotificationActionListT = [
             ('subscription-accept', self._on_subscription_accept, 'as'),
             ('subscription-deny', self._on_subscription_deny, 's'),
+            ('subscription-deny-all', self._on_subscription_deny_all, None),
             ('subscription-block', self._on_subscription_block, 's'),
             ('subscription-report', self._on_subscription_report, 's'),
         ]
         for action in actions:
             action_name, func, typ = action
+            if typ is not None:
+                typ = GLib.VariantType.new(typ)
             act = Gio.SimpleAction.new(
-                f'{action_name}-{self._account}', GLib.VariantType.new(typ))
+                f'{action_name}-{self._account}', typ)
             act.connect('activate', func)
             app.window.add_action(act)
 
@@ -81,30 +93,31 @@ class NotificationManager(Gtk.ListBox):
         online = app.account_is_connected(self._account)
         blocking_support = self._client.get_module('Blocking').supported
 
-        sub_accept = app.window.lookup_action(
+        sub_accept = app.window.get_action(
             f'subscription-accept-{self._account}')
-        assert sub_accept is not None
         sub_accept.set_enabled(online)
 
-        sub_deny = app.window.lookup_action(
+        sub_deny = app.window.get_action(
             f'subscription-deny-{self._account}')
-        assert sub_deny is not None
         sub_deny.set_enabled(online)
 
-        sub_block = app.window.lookup_action(
+        sub_deny_all = app.window.get_action(
+            f'subscription-deny-all-{self._account}')
+        sub_deny_all.set_enabled(online)
+
+        sub_block = app.window.get_action(
             f'subscription-block-{self._account}')
-        assert sub_block is not None
         sub_block.set_enabled(online and blocking_support)
 
-        sub_report = app.window.lookup_action(
+        sub_report = app.window.get_action(
             f'subscription-report-{self._account}')
-        assert sub_report is not None
         sub_report.set_enabled(online and blocking_support)
 
     def _remove_actions(self) -> None:
         actions = [
             'subscription-accept',
             'subscription-deny',
+            'subscription-deny-all',
             'subscription-block',
             'subscription-report',
         ]
@@ -127,6 +140,7 @@ class NotificationManager(Gtk.ListBox):
         self._client.get_module('Presence').subscribed(jid)
         jid = JID.from_string(jid)
         contact = self._client.get_module('Contacts').get_contact(jid)
+        assert isinstance(contact, BareContact)
         if not contact.is_in_roster:
             open_window('AddContact', account=self._account,
                         jid=jid, nick=nickname or contact.name)
@@ -165,10 +179,21 @@ class NotificationManager(Gtk.ListBox):
         if row is not None:
             row.destroy()
 
+    def _on_subscription_deny_all(self,
+                                  _action: Gio.SimpleAction,
+                                  _param: GLib.Variant
+                                  ) -> None:
+
+        for row in cast(list[NotificationRow], self.get_children()):
+            if row.type != 'subscribe':
+                continue
+            self._deny_request(row.jid)
+            row.destroy()
+
     def _deny_request(self, jid: str) -> None:
         self._client.get_module('Presence').unsubscribed(jid)
 
-    def _get_notification_row(self, jid: str) -> Optional[NotificationRow]:
+    def _get_notification_row(self, jid: str) -> NotificationRow | None:
         rows = cast(list[NotificationRow], self.get_children())
         for row in rows:
             if row.jid == jid:
@@ -192,6 +217,7 @@ class NotificationManager(Gtk.ListBox):
             if not nick:
                 contact = self._client.get_module('Contacts').get_contact(
                     event.jid)
+                assert isinstance(contact, BareContact)
                 nick = contact.name
             text = _('%s asks you to share your status') % nick
 
@@ -217,6 +243,7 @@ class NotificationManager(Gtk.ListBox):
 
             contact = self._client.get_module('Contacts').get_contact(
                 event.jid)
+            assert isinstance(contact, BareContact)
             text = _('%s stopped sharing their status') % contact.name
 
             app.ged.raise_event(
@@ -233,17 +260,20 @@ class NotificationManager(Gtk.ListBox):
         if row is not None:
             return
 
-        new_row = InvitationReceivedRow(self._account, event)
+        jid = event.from_.bare
+        client = app.get_client(event.account)
+        muc_contact = client.get_module('Contacts').get_contact(event.muc)
+        assert isinstance(muc_contact, GroupchatContact)
+
+        new_row = InvitationReceivedRow(self._account, event, muc_contact)
         new_row.connect('destroy', self._on_row_destroy)
         self.add(new_row)
         self.update_unread_count()
 
-        jid = event.from_.bare
-        client = app.get_client(event.account)
-        contact = client.get_module('Contacts').get_contact(event.muc)
-        if (contact.muc_context == 'private' and
+        if (muc_contact.muc_context == 'private' and
                 not event.muc.bare_match(event.from_)):
             contact = self._client.get_module('Contacts').get_contact(jid)
+            assert isinstance(contact, BareContact)
             text = _('%(contact)s invited you to %(chat)s') % {
                 'contact': contact.name,
                 'chat': event.info.muc_name}
@@ -289,10 +319,16 @@ class NotificationRow(Gtk.ListBoxRow):
         label.set_max_width_chars(30)
         return label
 
-    def _generate_avatar_image(self, jid: Union[str, JID]) -> Gtk.Image:
+    def _generate_avatar_image(self, jid: str | JID) -> Gtk.Image:
         contact = self._client.get_module('Contacts').get_contact(jid)
-        surface = contact.get_avatar(
-            AvatarSize.ROSTER, self.get_scale_factor(), add_show=False)
+        assert isinstance(
+            contact, BareContact | GroupchatContact | GroupchatParticipant)
+        if isinstance(contact, GroupchatContact):
+            surface = contact.get_avatar(
+                AvatarSize.ROSTER, self.get_scale_factor())
+        else:
+            surface = contact.get_avatar(
+                AvatarSize.ROSTER, self.get_scale_factor(), add_show=False)
         image = Gtk.Image.new_from_surface(surface)
         image.set_valign(Gtk.Align.CENTER)
         return image
@@ -303,7 +339,7 @@ class SubscriptionRequestRow(NotificationRow):
                  account: str,
                  jid: str,
                  text: str,
-                 user_nick: Optional[str] = None
+                 user_nick: str | None = None
                  ) -> None:
         NotificationRow.__init__(self, account, jid)
         self.type = 'subscribe'
@@ -361,6 +397,7 @@ class UnsubscribedRow(NotificationRow):
         self.grid.attach(image, 1, 1, 1, 2)
 
         contact = self._client.get_module('Contacts').get_contact(jid)
+        assert isinstance(contact, BareContact)
         nick_markup = f'<b>{contact.name}</b>'
         nick_label = self._generate_label()
         nick_label.set_tooltip_markup(nick_markup)
@@ -396,9 +433,13 @@ class UnsubscribedRow(NotificationRow):
 
 
 class InvitationReceivedRow(NotificationRow):
-    def __init__(self, account: str, event: MucInvitation) -> None:
+    def __init__(self,
+                 account: str,
+                 event: MucInvitation,
+                 muc_contact: GroupchatContact) -> None:
         NotificationRow.__init__(self, account, str(event.muc))
         self.type = 'invitation-received'
+        muc_contact.connect('room-joined', self._on_room_joined)
 
         self._event = event
 
@@ -411,11 +452,10 @@ class InvitationReceivedRow(NotificationRow):
         title_label.get_style_context().add_class('bold')
         self.grid.attach(title_label, 2, 1, 1, 1)
 
-        client = app.get_client(event.account)
-        contact = client.get_module('Contacts').get_contact(event.muc)
-        if (contact.muc_context == 'private' and
+        if (muc_contact.muc_context == 'private' and
                 not event.muc.bare_match(event.from_)):
             contact = self._client.get_module('Contacts').get_contact(jid)
+            assert isinstance(contact, BareContact)
             invitation_text = _('%(contact)s invited you to %(chat)s') % {
                 'contact': contact.name,
                 'chat': event.info.muc_name}
@@ -444,7 +484,7 @@ class InvitationReceivedRow(NotificationRow):
         self.show_all()
 
     def _on_show_invitation(self, _button: Gtk.Button) -> None:
-        open_window('GroupChatInvitation',
+        open_window('GroupChatInvitationDialog',
                     account=self._account,
                     event=self._event)
         self.destroy()
@@ -454,6 +494,10 @@ class InvitationReceivedRow(NotificationRow):
             self.jid, self._event.from_)
         self.destroy()
 
+    def _on_room_joined(self,
+                        contact: GroupchatContact,
+                        signal_name: str) -> None:
+        self.destroy()
 
 class InvitationDeclinedRow(NotificationRow):
     def __init__(self, account: str, event: MucDecline) -> None:
@@ -471,6 +515,7 @@ class InvitationDeclinedRow(NotificationRow):
         self.grid.attach(title_label, 2, 1, 1, 1)
 
         contact = self._client.get_module('Contacts').get_contact(jid)
+        assert isinstance(contact, BareContact)
         muc_name = get_groupchat_name(self._client, event.muc)
         invitation_text = _('%(contact)s declined your invitation '
                             'to %(chat)s') % {
