@@ -3,33 +3,20 @@
 #
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
 from typing import Any
-from typing import Optional
-from typing import Union
 
 import logging
 import math
 import sys
 import textwrap
+from datetime import datetime
 from functools import lru_cache
 from functools import wraps
 from importlib import import_module
-from io import BytesIO
-from pathlib import Path
 from re import Match
 
 import cairo
@@ -44,27 +31,32 @@ from nbxmpp import JID
 from nbxmpp import util as nbxmpp_util
 from nbxmpp.structs import LocationData
 from nbxmpp.structs import TuneData
-from PIL import Image
 
 from gajim.common import app
 from gajim.common import configpaths
 from gajim.common import types
+from gajim.common.const import AvatarSize
 from gajim.common.const import Display
 from gajim.common.const import LOCATION_DATA
-from gajim.common.const import StyleAttr
 from gajim.common.ged import EventHelper as CommonEventHelper
+from gajim.common.helpers import format_idle_time
 from gajim.common.helpers import URL_REGEX
 from gajim.common.i18n import _
+from gajim.common.modules.contacts import BareContact
+from gajim.common.modules.contacts import GroupchatContact
 from gajim.common.modules.contacts import GroupchatParticipant
+from gajim.common.storage.archive import models as mod
+from gajim.common.storage.archive.const import ChatDirection
+from gajim.common.storage.archive.const import MessageType
 from gajim.common.structs import VariantMixin
 from gajim.common.styling import PlainBlock
 
-from .const import WINDOW_MODULES
+from gajim.gtk.const import WINDOW_MODULES
 
-log = logging.getLogger('gajim.gui.util')
+log = logging.getLogger('gajim.gtk.util')
 
 
-MenuValueT = Union[None, str, GLib.Variant, VariantMixin]
+MenuValueT = None | str | GLib.Variant | VariantMixin
 MenuItemListT = list[tuple[str, str, MenuValueT]]
 
 
@@ -79,8 +71,8 @@ def icon_exists(name: str) -> bool:
 
 def load_icon_info(icon_name: str,
                    size: int,
-                   scale: Optional[int],
-                   flags: Gtk.IconLookupFlags) -> Optional[Gtk.IconInfo]:
+                   scale: int | None,
+                   flags: Gtk.IconLookupFlags) -> Gtk.IconInfo | None:
 
     if scale is None:
         scale = app.window.get_scale_factor()
@@ -106,26 +98,52 @@ def load_icon_info(icon_name: str,
 def load_icon_surface(
         icon_name: str,
         size: int = 16,
-        scale: Optional[int] = None,
+        scale: int | None = None,
         flags: Gtk.IconLookupFlags = Gtk.IconLookupFlags.FORCE_SIZE
-) -> Optional[cairo.ImageSurface]:
+) -> cairo.ImageSurface | None:
 
     icon_info = load_icon_info(icon_name, size, scale, flags)
     if icon_info is None:
         return None
-    return icon_info.load_surface(None)
+
+    try:
+        surface = icon_info.load_surface(None)
+    except GLib.Error as e:
+        log.error(
+            'Error loading icon surface for %s, %s, %s: %s',
+            icon_name,
+            size,
+            scale,
+            e
+        )
+        return None
+
+    return surface
 
 
 def load_icon_pixbuf(icon_name: str,
                      size: int = 16,
-                     scale: Optional[int] = None,
+                     scale: int | None = None,
                      flags: Gtk.IconLookupFlags = Gtk.IconLookupFlags.FORCE_SIZE
-                     ) -> Optional[GdkPixbuf.Pixbuf]:
+                     ) -> GdkPixbuf.Pixbuf | None:
 
     icon_info = load_icon_info(icon_name, size, scale, flags)
     if icon_info is None:
         return None
-    return icon_info.load_icon()
+
+    try:
+        icon = icon_info.load_icon()
+    except GLib.Error as e:
+        log.error(
+            'Error loading icon surface for %s, %s, %s: %s',
+            icon_name,
+            size,
+            scale,
+            e
+        )
+        return None
+
+    return icon
 
 
 def get_app_icon_list(scale_widget: Gtk.Widget) -> list[GdkPixbuf.Pixbuf]:
@@ -139,7 +157,7 @@ def get_app_icon_list(scale_widget: Gtk.Widget) -> list[GdkPixbuf.Pixbuf]:
 
 
 def get_icon_name(name: str,
-                  transport: Optional[str] = None) -> str:
+                  transport: str | None = None) -> str:
     if transport is not None:
         return f'{transport}-{name}'
 
@@ -166,7 +184,7 @@ def get_total_screen_geometry() -> tuple[int, int]:
     display = Gdk.Display.get_default()
     assert display is not None
     monitors = display.get_n_monitors()
-    for num in range(0, monitors):
+    for num in range(monitors):
         monitor = display.get_monitor(num)
         assert monitor is not None
         geometry = monitor.get_geometry()
@@ -226,7 +244,7 @@ def restore_main_window_position() -> None:
                 app.settings.get('mainwin_y_position'))
 
 
-def get_source_view_style_scheme() -> Optional[GtkSource.StyleScheme]:
+def get_source_view_style_scheme() -> GtkSource.StyleScheme | None:
     style_scheme_manager = GtkSource.StyleSchemeManager.get_default()
     if app.css_config.prefer_dark:
         return style_scheme_manager.get_scheme('solarized-dark')
@@ -257,10 +275,16 @@ def get_completion_liststore(entry: Gtk.Entry) -> Gtk.ListStore:
 def get_cursor(name: str) -> Gdk.Cursor:
     display = Gdk.Display.get_default()
     assert display is not None
-    cursor = Gdk.Cursor.new_from_name(display, name)
-    if cursor is not None:
-        return cursor
-    return Gdk.Cursor.new_from_name(display, 'default')
+    try:
+        cursor = Gdk.Cursor.new_from_name(display, name)
+        if cursor is not None:
+            return cursor
+    except TypeError as e:
+        log.exception(e)
+
+    cursor = Gdk.Cursor.new_from_name(display, 'default')
+    assert cursor is not None
+    return cursor
 
 
 def scroll_to_end(widget: Gtk.ScrolledWindow) -> bool:
@@ -273,7 +297,7 @@ def scroll_to_end(widget: Gtk.ScrolledWindow) -> bool:
         bool: The return value is False so it can be used with GLib.idle_add.
     '''
     adj_v = widget.get_vadjustment()
-    if adj_v is None:
+    if adj_v is None:  # pyright: ignore
         # This can happen when the Widget is already destroyed when called
         # from GLib.idle_add
         return False
@@ -323,7 +347,7 @@ def convert_rgba_to_hex(rgba: Gdk.RGBA) -> str:
     red = int(rgba.red * 255)
     green = int(rgba.green * 255)
     blue = int(rgba.blue * 255)
-    return '#%02x%02x%02x' % (red, green, blue)
+    return f'#{red:02x}{green:02x}{blue:02x}'
 
 
 def convert_rgb_to_hex(rgb_string: str) -> str:
@@ -351,6 +375,7 @@ def make_rgba(color_string: str) -> Gdk.RGBA:
 
 
 def get_monitor_scale_factor() -> int:
+    # Does not work on Wayland
     display = Gdk.Display.get_default()
     assert display is not None
     monitor = display.get_primary_monitor()
@@ -360,7 +385,7 @@ def get_monitor_scale_factor() -> int:
     return monitor.get_scale_factor()
 
 
-def get_primary_accel_mod() -> Optional[Gdk.ModifierType]:
+def get_primary_accel_mod() -> Gdk.ModifierType | None:
     '''
     Returns the primary Gdk.ModifierType modifier.
     cmd on osx, ctrl everywhere else.
@@ -411,7 +436,7 @@ def format_tune(data: TuneData) -> str:
                                           'source': source}
 
 
-def get_account_tune_icon_name(account: str) -> Optional[str]:
+def get_account_tune_icon_name(account: str) -> str | None:
     client = app.get_client(account)
     tune = client.get_module('UserTune').get_current_tune()
     return None if tune is None else 'audio-x-generic'
@@ -428,19 +453,18 @@ def format_location(location: LocationData) -> str:
         tag = LOCATION_DATA.get(attr)
         if tag is None:
             continue
-        location_string += '\n<b>%(tag)s</b>: %(text)s' % {
-            'tag': tag.capitalize(), 'text': text}
+        location_string += f'\n<b>{tag.capitalize()}</b>: {text}'
 
     return location_string.strip()
 
 
-def get_account_location_icon_name(account: str) -> Optional[str]:
+def get_account_location_icon_name(account: str) -> str | None:
     client = app.get_client(account)
     location = client.get_module('UserLocation').get_current_location()
     return None if location is None else 'applications-internet'
 
 
-def format_eta(time_: Union[int, float]) -> str:
+def format_eta(time_: int | float) -> str:
     times = {'minutes': 0, 'seconds': 0}
     time_ = int(time_)
     times['seconds'] = time_ % 60
@@ -456,12 +480,12 @@ def format_fingerprint(fingerprint: str) -> str:
     wordsize = fplen // 8
     buf = ''
     for char in range(0, fplen, wordsize):
-        buf += '{0} '.format(fingerprint[char:char + wordsize])
+        buf += f'{fingerprint[char:char + wordsize]} '
     buf = textwrap.fill(buf, width=36)
     return buf.rstrip().upper()
 
 
-def find_widget(name: str, container: Gtk.Container) -> Optional[Gtk.Widget]:
+def find_widget(name: str, container: Gtk.Container) -> Gtk.Widget | None:
     for child in container.get_children():
         if Gtk.Buildable.get_name(child) == name:
             return child
@@ -478,6 +502,13 @@ class MultiLineLabel(Gtk.Label):
         self.set_single_line_mode(False)
         self.set_selectable(True)
 
+    # make sure full width is used when fitting into GtkGrid
+    def do_get_preferred_width(self):
+        layout = self.get_layout()
+        layout.set_width(-1)
+        width, _ = layout.get_pixel_size()
+        return width, width
+
 
 class MaxWidthComboBoxText(Gtk.ComboBoxText):
     def __init__(self, *args: Any, **kwargs: Any) -> None:
@@ -548,98 +579,57 @@ def add_css_to_widget(widget: Any, css: str) -> None:
                          Gtk.STYLE_PROVIDER_PRIORITY_USER)
 
 
-def get_pixbuf_from_data(file_data: bytes) -> Optional[GdkPixbuf.Pixbuf]:
-    # TODO: This already exists in preview_helpery pixbuf_from_data
-    '''
-    Get image data and returns GdkPixbuf.Pixbuf
-    '''
-    pixbufloader = GdkPixbuf.PixbufLoader()
-    try:
-        pixbufloader.write(file_data)
-        pixbufloader.close()
-        pixbuf = pixbufloader.get_pixbuf()
-    except GLib.Error:
-        pixbufloader.close()
+def get_contact_name_for_message(
+        db_row: mod.Message,
+        contact: types.ChatContactT
+    ) -> str:
 
-        log.warning('loading avatar using pixbufloader failed, trying to '
-                    'convert avatar image using pillow')
-        try:
-            avatar = Image.open(BytesIO(file_data)).convert('RGBA')
-            array = GLib.Bytes.new(avatar.tobytes())  # pyright: ignore
-            width, height = avatar.size
-            pixbuf = GdkPixbuf.Pixbuf.new_from_bytes(
-                array, GdkPixbuf.Colorspace.RGB,
-                True, 8, width, height, width * 4)
-        except Exception:
-            log.warning('Could not use pillow to convert avatar image, '
-                        'image cannot be displayed', exc_info=True)
-            return None
+    if isinstance(contact, BareContact) and contact.is_self:
+        return _('Me')
 
-    return pixbuf
+    if db_row.type == MessageType.CHAT:
+        if db_row.direction == ChatDirection.INCOMING:
+            return contact.name
+        return app.nicks[contact.account]
+
+    elif db_row.type == MessageType.GROUPCHAT:
+        resource = db_row.resource
+        if resource is None:
+            # Fall back to MUC name if contact name is None
+            # (may be the case for service messages from the MUC)
+            return contact.name
+        return resource
+
+    elif db_row.type == MessageType.PM:
+        resource = db_row.resource
+        assert resource is not None
+        return resource
+
+    else:
+        raise ValueError
 
 
-def scale_with_ratio(size: int, width: int, height: int) -> tuple[int, int]:
-    if height == width:
-        return size, size
-    if height > width:
-        ratio = height / float(width)
-        return int(size / ratio), size
+def get_avatar_for_message(
+        db_row: mod.Message,
+        contact: types.ChatContactT,
+        scale: int,
+        size: AvatarSize
+    ) -> cairo.ImageSurface | None:
 
-    ratio = width / float(height)
-    return size, int(size / ratio)
+    if isinstance(contact, GroupchatContact):
+        name = get_contact_name_for_message(db_row, contact)
+        resource_contact = contact.get_resource(name)
+        return resource_contact.get_avatar(
+            size, scale, add_show=False)
 
+    if db_row.direction == ChatDirection.OUTGOING:
+        client = app.get_client(contact.account)
+        self_contact = client.get_module('Contacts').get_contact(
+            client.get_own_jid().bare)
+        assert isinstance(self_contact, BareContact)
+        return self_contact.get_avatar(size, scale, add_show=False)
 
-def scale_pixbuf(pixbuf: GdkPixbuf.Pixbuf,
-                 size: int) -> Optional[GdkPixbuf.Pixbuf]:
-    width, height = scale_with_ratio(size,
-                                     pixbuf.get_width(),
-                                     pixbuf.get_height())
-    return pixbuf.scale_simple(width, height,
-                               GdkPixbuf.InterpType.BILINEAR)
-
-
-def scale_pixbuf_from_data(data: bytes,
-                           size: int
-                           ) -> Optional[GdkPixbuf.Pixbuf]:
-    pixbuf = get_pixbuf_from_data(data)
-    assert pixbuf is not None
-    return scale_pixbuf(pixbuf, size)
-
-
-def load_pixbuf(path: Union[str, Path],
-                size: Optional[int] = None
-                ) -> Optional[GdkPixbuf.Pixbuf]:
-    try:
-        if size is None:
-            return GdkPixbuf.Pixbuf.new_from_file(str(path))
-        return GdkPixbuf.Pixbuf.new_from_file_at_scale(
-            str(path), size, size, True)
-
-    except GLib.Error:
-        try:
-            with open(path, 'rb') as im_handle:
-                img = Image.open(im_handle)
-                avatar = img.convert('RGBA')
-        except (NameError, OSError):
-            log.warning('Pillow convert failed: %s', path)
-            log.debug('Error', exc_info=True)
-            return None
-
-        array = GLib.Bytes.new(avatar.tobytes())  # pyright: ignore
-        width, height = avatar.size
-        pixbuf = GdkPixbuf.Pixbuf.new_from_bytes(
-            array, GdkPixbuf.Colorspace.RGB, True,
-            8, width, height, width * 4)
-        if size is not None:
-            width, height = scale_with_ratio(size, width, height)
-            return pixbuf.scale_simple(width,
-                                       height,
-                                       GdkPixbuf.InterpType.BILINEAR)
-        return pixbuf
-
-    except RuntimeError as error:
-        log.warning('Loading pixbuf failed: %s', error)
-        return None
+    return contact.get_avatar(size, scale, add_show=False)
 
 
 def get_thumbnail_size(pixbuf: GdkPixbuf.Pixbuf, size: int) -> tuple[int, int]:
@@ -650,28 +640,28 @@ def get_thumbnail_size(pixbuf: GdkPixbuf.Pixbuf, size: int) -> tuple[int, int]:
     if image_width > image_height:
         if image_width > size:
             image_height = math.ceil(
-                (size / float(image_width) * image_height))
+                size / float(image_width) * image_height)
             image_width = int(size)
     else:
         if image_height > size:
             image_width = math.ceil(
-                (size / float(image_height) * image_width))
+                size / float(image_height) * image_width)
             image_height = int(size)
 
     return image_width, image_height
 
 
-def make_href_markup(string: str) -> str:
-    url_color = app.css_config.get_value('.gajim-url', StyleAttr.COLOR)
-    assert isinstance(url_color, str)
-    color = convert_rgb_to_hex(url_color)
+def make_href_markup(string: str | None) -> str:
+    if not string:
+        return ''
+
+    string = GLib.markup_escape_text(string)
 
     def _to_href(match: Match[str]) -> str:
         url = match.group()
         if '://' not in url:
-            url = 'https://' + url
-        return '<a href="%s"><span foreground="%s">%s</span></a>' % (
-            url, color, match.group())
+            url = f'https://{url}'
+        return f'<a href="{url}">{match.group()}</a>'
 
     return URL_REGEX.sub(_to_href, string)
 
@@ -686,9 +676,9 @@ def get_app_windows(account: str) -> list[Gtk.Window]:
 
 
 def get_app_window(name: str,
-                   account: Optional[str] = None,
-                   jid: Optional[Union[str, JID]] = None
-                   ) -> Optional[Gtk.Window]:
+                   account: str | None = None,
+                   jid: str | JID | None = None
+                   ) -> Gtk.Window | None:
     for win in app.app.get_windows():
         if type(win).__name__ != name:
             continue
@@ -794,8 +784,48 @@ def wrap_with_event_box(klass: Any) -> Any:
     return klass_wrapper
 
 
+class GroupBadge(Gtk.Label):
+    def __init__(self, group: str) -> None:
+        Gtk.Label.__init__(
+            self,
+            ellipsize=Pango.EllipsizeMode.END,
+            no_show_all=True,
+            halign=Gtk.Align.END,
+            valign=Gtk.Align.CENTER,
+            hexpand=True,
+            tooltip_text=group,
+        )
+        self.set_size_request(50, -1)
+        self.get_style_context().add_class('group')
+        self.set_text(group)
+        self.show()
+
+
+class IdleBadge(Gtk.Label):
+    def __init__(self, idle: datetime | None = None) -> None:
+        Gtk.Label.__init__(
+            self,
+            halign=Gtk.Align.START,
+            hexpand=True,
+            ellipsize=Pango.EllipsizeMode.NONE,
+            no_show_all=True,
+        )
+        self.set_size_request(50, -1)
+        self.get_style_context().add_class('dim-label')
+        self.get_style_context().add_class('small-label')
+        if idle is not None:
+            self.set_idle(idle)
+
+        self.show()
+
+    def set_idle(self, idle: datetime) -> None:
+        self.set_text(_('Last seen: %s') % format_idle_time(idle))
+        format_string = app.settings.get('date_time_format')
+        self.set_tooltip_text(idle.strftime(format_string))
+
+
 class AccountBadge(Gtk.Label):
-    def __init__(self, account: Optional[str] = None) -> None:
+    def __init__(self, account: str | None = None) -> None:
         Gtk.Label.__init__(self)
         self.set_ellipsize(Pango.EllipsizeMode.END)
         self.set_max_width_chars(12)
@@ -829,7 +859,7 @@ class AccountBadge(Gtk.Label):
     def _on_account_label_changed(self,
                                   _value: str,
                                   _setting: str,
-                                  account: Optional[str],
+                                  account: str | None,
                                   *args: Any
                                   ) -> None:
 
@@ -853,7 +883,7 @@ _grapheme_buffer = Gtk.TextBuffer()
 def get_first_graphemes(text: str, n: int) -> str:
     # This should be possible with lower-level APIs like Pango.break_* or
     # Pango.get_log_attrs, but their Python bindings seem totally broken.
-    # The re-use of one global buffer is to mitigate very probable memory leaks.
+    # The reuse of one global buffer is to mitigate very probable memory leaks.
     _grapheme_buffer.set_text(text)
     cursor = _grapheme_buffer.get_start_iter()
     cursor.forward_cursor_positions(n)
@@ -881,7 +911,7 @@ def get_style_attribute_with_name(name: str) -> Pango.Attribute:
     raise ValueError('unknown attribute %s' % name)
 
 
-def get_key_theme() -> Optional[str]:
+def get_key_theme() -> str | None:
     settings = Gtk.Settings.get_default()
     if settings is None:
         return None
@@ -889,7 +919,7 @@ def get_key_theme() -> Optional[str]:
 
 
 def make_menu_item(label: str,
-                   action: Optional[str] = None,
+                   action: str | None = None,
                    value: MenuValueT = None) -> Gio.MenuItem:
 
     item = Gio.MenuItem.new(label)
@@ -923,7 +953,7 @@ class GajimMenu(Gio.Menu):
     def add_item(self,
                  label: str,
                  action: str,
-                 value: Optional[MenuValueT] = None) -> None:
+                 value: MenuValueT | None = None) -> None:
         item = make_menu_item(label, action, value)
         self.append_item(item)
 
@@ -951,9 +981,9 @@ class GdkRectangle(Gdk.Rectangle):
 class GajimPopover(Gtk.Popover):
     def __init__(self,
                  menu: Gio.MenuModel,
-                 relative_to: Optional[Gtk.Widget] = None,
+                 relative_to: Gtk.Widget | None = None,
                  position: Gtk.PositionType = Gtk.PositionType.RIGHT,
-                 event: Optional[Gdk.EventButton] = None) -> None:
+                 event: Gdk.EventButton | None = None) -> None:
 
         Gtk.Popover.__init__(self)
 
